/**
 * TextRad Dictation Extension - Service Worker (Background Script)
 * 
 * Responsibilities:
 * - Managing extension lifecycle
 * - Coordinating communication between components
 * - Tab management and detection
 * - State persistence
 */

// Track TextRad tabs
let textradTabs = {};
let activeTabId = null;

// Track current state
let currentState = {
  recording: false,
  transcribing: false,
  lastTranscription: null,
  lastTranscriptionTimestamp: null
};

// Background recording has been disabled - no offscreen document needed

/**
 * Initialize the service worker
 */
function initialize() {
  console.log('TextRad Dictation Extension service worker initialized');
  
  // Load state from storage
  chrome.storage.local.get(['textradTabs', 'currentState'], (data) => {
    if (data.textradTabs) {
      textradTabs = data.textradTabs;
    }
    
    if (data.currentState) {
      currentState = data.currentState;
    }
    
    // Verify tabs still exist and are TextRad tabs
    verifyStoredTabs();
  });
  
  // Setup listeners
  setupListeners();
}

/**
 * Verify that stored tabs still exist
 */
async function verifyStoredTabs() {
  const validTabs = {};
  
  for (const tabId in textradTabs) {
    try {
      // Check if tab still exists
      const tab = await chrome.tabs.get(parseInt(tabId));
      if (tab && isTextRadUrl(tab.url)) {
        validTabs[tabId] = textradTabs[tabId];
      }
    } catch (error) {
      // Tab no longer exists, ignore error
    }
  }
  
  // Update stored tabs
  textradTabs = validTabs;
  saveState();
}

/**
 * Check if URL is a TextRad URL
 */
function isTextRadUrl(url) {
  return url && (
    url.includes('textrad.com') || 
    url.includes('textrad.in') || 
    url.includes('localhost')
  );
}

/**
 * Set up message and event listeners
 */
function setupListeners() {
  // Listen for messages from popup and content scripts
  chrome.runtime.onMessage.addListener(handleMessage);
  
  // Listen for tab updates
  chrome.tabs.onUpdated.addListener(handleTabUpdated);
  
  // Listen for tab removal
  chrome.tabs.onRemoved.addListener(handleTabRemoved);
  
  // Listen for active tab changes
  chrome.tabs.onActivated.addListener(handleTabActivated);
}

/**
 * Handle messages from popup and content scripts
 */
function handleMessage(message, sender, sendResponse) {
  console.log('Received message:', message);
  
  // Handle different message types
  switch (message.action) {
    case 'START_RECORDING':
      handleStartRecording(sendResponse);
      break;
      
    case 'STOP_RECORDING':
      handleStopRecording(sendResponse);
      break;
      
    case 'MAINTAIN_RECORDING':
      // Handler to maintain recording when popup is closed
      handleMaintainRecording(message.inProgress, sendResponse);
      break;
      
    case 'SET_VIEW_MODE':
      // Save view mode in storage
      chrome.storage.local.set({ viewMode: message.mode });
      sendResponse({ success: true });
      break;
      
    case 'SET_TRANSCRIPTION':
      handleSetTranscription(message.transcription, sendResponse);
      break;
      
    case 'SEND_TRANSCRIPTION_TO_TAB':
      handleSendTranscriptionToTab(message.tabId, message.transcription, sendResponse);
      return true; // Keep the message channel open for async response
      
    case 'FIND_TEXTRAD_TABS':
      findTextRadTabs(sendResponse);
      return true; // Keep the message channel open for async response
      
    case 'CONNECTED_TO_TEXTRAD':
      handleTextRadTabConnected(sender.tab.id, sendResponse);
      break;
      
    case 'GET_STATE':
      sendResponse({ 
        state: currentState, 
        tabs: textradTabs,
        recordingInBackground: currentState.recording
      });
      break;
      
    case 'GET_FIREBASE_CONFIG':
      // If we have stored Firebase config, return it
      chrome.storage.local.get(['firebaseConfig'], (data) => {
        sendResponse({ config: data.firebaseConfig || null });
      });
      return true; // Keep the message channel open for async response
  }
  
  // Return false to close the message channel
  return false;
}

/**
 * Handle tab updates
 */
function handleTabUpdated(tabId, changeInfo, tab) {
  // Check if this is a TextRad tab
  if (changeInfo.status === 'complete' && isTextRadUrl(tab.url)) {
    // Add to tracked tabs if not already tracked
    if (!textradTabs[tabId]) {
      textradTabs[tabId] = {
        url: tab.url,
        title: tab.title,
        lastConnected: Date.now()
      };
      saveState();
    }
  }
}

/**
 * Handle tab removal
 */
function handleTabRemoved(tabId) {
  // Remove from tracked tabs
  if (textradTabs[tabId]) {
    delete textradTabs[tabId];
    saveState();
  }
  
  // Reset active tab if it was this one
  if (activeTabId === tabId) {
    activeTabId = null;
  }
}

/**
 * Handle tab activation
 */
function handleTabActivated(activeInfo) {
  // Check if this is a TextRad tab
  if (textradTabs[activeInfo.tabId]) {
    activeTabId = activeInfo.tabId;
  }
}

/**
 * Handle TextRad tab connection
 */
function handleTextRadTabConnected(tabId, sendResponse) {
  // Update tracked tabs
  chrome.tabs.get(tabId, (tab) => {
    if (chrome.runtime.lastError) {
      console.error('Error getting tab:', chrome.runtime.lastError);
      return;
    }
    
    textradTabs[tabId] = {
      url: tab.url,
      title: tab.title,
      lastConnected: Date.now(),
      connected: true
    };
    
    saveState();
    sendResponse({ success: true });
  });
}

/**
 * Handle starting recording
 */
function handleStartRecording(sendResponse) {
  currentState.recording = true;
  currentState.transcribing = false;
  saveState();
  
  // Update extension badge
  chrome.action.setBadgeText({ text: 'REC' });
  chrome.action.setBadgeBackgroundColor({ color: '#FF0000' });
  
  sendResponse({ success: true });
}

/**
 * Handle stopping recording
 */
function handleStopRecording(sendResponse) {
  currentState.recording = false;
  currentState.transcribing = true;
  saveState();
  
  // Update extension badge
  chrome.action.setBadgeText({ text: 'TRS' });
  chrome.action.setBadgeBackgroundColor({ color: '#0000FF' });
  
  sendResponse({ success: true });
}

/**
 * Handle maintaining recording in background when popup closes
 * Now we create a detached window to keep the recording alive
 */
function handleMaintainRecording(inProgress, sendResponse) {
  if (inProgress) {
    console.log('Maintaining recording in progress - keeping recording active');
    
    // Update state to indicate we're recording
    currentState.recording = true;
    saveState();
    
    // Update badge to show recording is in progress
    chrome.action.setBadgeText({ text: 'REC' });
    chrome.action.setBadgeBackgroundColor({ color: '#FF0000' });
    
    // Make badge blink to indicate background recording
    startBadgeBlinking();
    
    // Open a detached mini window to continue recording
    openRecordingWindow();
  } else {
    // Reset recording state
    currentState.recording = false;
    saveState();
    
    // Clear any recording badge
    chrome.action.setBadgeText({ text: '' });
    
    // Stop any blinking badge
    stopBadgeBlinking();
  }
  
  // Response is optional for this handler
  if (sendResponse) {
    sendResponse({ success: true });
  }
}

/**
 * Open a detached recording window to continue recording
 */
function openRecordingWindow() {
  // Get the extension's popup URL
  const popupURL = chrome.runtime.getURL('popup/popup.html');
  
  // Check if we already have a recording window open
  if (recordingWindowId) {
    // Focus the existing window
    chrome.windows.update(recordingWindowId, { focused: true });
    return;
  }
  
  // Create a new window with the popup content that's sized perfectly for the mini mode
  chrome.windows.create({
    url: popupURL + '?viewMode=mini',
    type: 'popup',
    width: 150,  // Mini width with margin
    height: 40,  // Mini height
    left: Math.round(screen.width - 100),
    top: 50,
    focused: false
  }, (window) => {
    if (window) {
      recordingWindowId = window.id;
      
      // Add a listener for when the window is closed
      chrome.windows.onRemoved.addListener((windowId) => {
        if (windowId === recordingWindowId) {
          recordingWindowId = null;
          
          // If window was closed, stop the recording
          currentState.recording = false;
          saveState();
          
          // Clear badge
          chrome.action.setBadgeText({ text: '' });
          stopBadgeBlinking();
        }
      });
    }
  });
}

// Track recording window
let recordingWindowId = null;

// Badge blinking for background recording
let badgeBlinkInterval = null;

/**
 * Start badge blinking animation to indicate background recording
 */
function startBadgeBlinking() {
  // Clear any existing interval
  if (badgeBlinkInterval) {
    clearInterval(badgeBlinkInterval);
  }
  
  // Set up blinking every 1 second
  let visible = true;
  badgeBlinkInterval = setInterval(() => {
    if (visible) {
      chrome.action.setBadgeText({ text: '' });
    } else {
      chrome.action.setBadgeText({ text: 'REC' });
    }
    visible = !visible;
  }, 1000);
}

/**
 * Stop badge blinking
 */
function stopBadgeBlinking() {
  if (badgeBlinkInterval) {
    clearInterval(badgeBlinkInterval);
    badgeBlinkInterval = null;
  }
  
  // Restore badge based on current state
  if (currentState.recording) {
    chrome.action.setBadgeText({ text: 'REC' });
  } else if (currentState.transcribing) {
    chrome.action.setBadgeText({ text: 'TRS' });
  } else {
    chrome.action.setBadgeText({ text: '' });
  }
}

/**
 * Handle setting transcription
 */
function handleSetTranscription(transcription, sendResponse) {
  currentState.transcribing = false;
  currentState.recording = false; // Reset recording state
  currentState.lastTranscription = transcription;
  currentState.lastTranscriptionTimestamp = Date.now();
  saveState();
  
  // Update extension badge
  chrome.action.setBadgeText({ text: '' });
  
  // Stop badge blinking if it was active
  stopBadgeBlinking();
  
  sendResponse({ success: true });
}

/**
 * Handle sending transcription to tab
 */
async function handleSendTranscriptionToTab(tabId, transcription, sendResponse) {
  try {
    // If no tabId provided, use active TextRad tab
    if (!tabId) {
      const tabs = await findTextRadTabs();
      if (tabs.length === 0) {
        throw new Error('No logged-in TextRad tabs found. Please log in to TextRad in your browser.');
      }
      tabId = tabs[0].id;
    }
    
    // Verify the tab is still valid
    try {
      const tab = await chrome.tabs.get(tabId);
      if (!tab || !isTextRadUrl(tab.url)) {
        throw new Error('The selected TextRad tab is no longer valid');
      }
    } catch (error) {
      throw new Error('Tab no longer exists or is not accessible');
    }
    
    // Ensure the content script is loaded by injecting it if needed
    try {
      // Check if the content script is already injected
      const pingResponse = await chrome.tabs.sendMessage(tabId, { action: 'PING' });
      console.log('Content script already exists in tab');
      
      // Check if the user is logged in
      if (pingResponse && pingResponse.loggedIn === false) {
        throw new Error('User is not logged in to TextRad. Please log in and go to TextRad Web App to use the plugin.');
      }
    } catch (error) {
      if (error.message.includes('not logged in')) {
        // Re-throw the login error
        throw error;
      }
      
      // Content script not found, inject it
      console.log('Content script not found in tab, injecting it now');
      await chrome.scripting.executeScript({
        target: { tabId: tabId },
        files: ['content/content-script.js']
      });
      
      // Wait a moment for content script to initialize
      await new Promise(resolve => setTimeout(resolve, 500));
      
      // Verify login state after injection
      try {
        const loginCheckResponse = await chrome.tabs.sendMessage(tabId, { action: 'CHECK_LOGIN' });
        if (loginCheckResponse && loginCheckResponse.loggedIn === false) {
          throw new Error('User is not logged in to TextRad. Please log in and go to TextRad Web App to use the plugin.');
        }
      } catch (error) {
        if (error.message.includes('not logged in')) {
          // Re-throw the login error
          throw error;
        }
        console.warn('Could not verify login state after injection:', error);
      }
    }
    
    // Send message to content script
    const response = await chrome.tabs.sendMessage(tabId, {
      action: 'INJECT_TRANSCRIPTION',
      transcription: transcription
    });
    
    if (response && response.success) {
      sendResponse({ success: true, tabId: tabId });
    } else {
      throw new Error('Failed to inject transcription. Ensure you are logged in to TextRad Web App.');
    }
  } catch (error) {
    console.error('Error sending transcription to tab:', error);
    sendResponse({ success: false, error: error.message });
  }
}

/**
 * Find all TextRad tabs
 */
async function findTextRadTabs(sendResponse) {
  try {
    // First get all tabs that match TextRad URLs
    const tabs = await chrome.tabs.query({ url: ['*://*.textrad.com/*', '*://*.textrad.in/*', '*://localhost/*'] });
    
    // Check if we found any tabs at all
    if (tabs.length === 0) {
      if (sendResponse) {
        sendResponse({ 
          success: true, 
          tabs: [], 
          hasTextRadTabs: false,
          notLoggedInCount: 0
        });
      }
      return [];
    }
    
    // Now filter to only those where user is logged in
    const loggedInTabs = [];
    const potentialTabs = {};
    const loggedInTabsMap = {};
    let notLoggedInCount = 0;
    
    for (const tab of tabs) {
      try {
        // Try to check login state for each tab
        const loginCheckResult = await checkTabLoginState(tab.id);
        
        if (loginCheckResult) {
          // User is logged in, add to our list
          loggedInTabs.push(tab);
          loggedInTabsMap[tab.id] = true;
          
          // Track this tab
          potentialTabs[tab.id] = {
            url: tab.url,
            title: tab.title,
            lastConnected: Date.now(),
            connected: true,
            loggedIn: true
          };
        } else {
          console.log(`Tab ${tab.id} is TextRad but user not logged in, skipping`);
          notLoggedInCount++;
          
          // Track this tab but mark as not logged in
          potentialTabs[tab.id] = {
            url: tab.url,
            title: tab.title,
            lastConnected: Date.now(),
            connected: false,
            loggedIn: false
          };
        }
      } catch (error) {
        console.log(`Error checking login state for tab ${tab.id}:`, error);
        notLoggedInCount++;
      }
    }
    
    // Update tracked tabs
    textradTabs = potentialTabs;
    saveState();
    
    if (sendResponse) {
      sendResponse({ 
        success: true, 
        tabs: loggedInTabs,
        loggedInTabs: loggedInTabsMap,
        hasTextRadTabs: true,
        notLoggedInCount: notLoggedInCount,
        error: loggedInTabs.length === 0 && notLoggedInCount > 0 ? 
              'TextRad tabs found but user not logged in' : null
      });
    }
    
    return loggedInTabs;
  } catch (error) {
    console.error('Error finding TextRad tabs:', error);
    if (sendResponse) {
      sendResponse({ success: false, error: error.message });
    }
    return [];
  }
}

/**
 * Check if the user is logged in for a specific tab
 */
async function checkTabLoginState(tabId) {
  try {
    // First, check if tab is on the TextRad domain
    let tab;
    try {
      tab = await chrome.tabs.get(tabId);
      if (!tab || !isTextRadUrl(tab.url)) {
        console.log(`Tab ${tabId} is not a TextRad URL`);
        return false;
      }
      
      // Don't make assumptions based on URL path - both main page and app page can be 
      // valid if the user is logged in. Let the content script determine login state.
    } catch (error) {
      console.log(`Error getting tab info for ${tabId}:`, error);
      return false;
    }
    
    // Determine if we're on the main site or app page
    const isMainSite = tab.url.includes('textrad.in/') === false || 
                     tab.url.endsWith('textrad.in/') || 
                     tab.url.includes('textrad.in/index') ||
                     tab.url.includes('textrad.in/?');
                     
    const isAppPage = tab.url.includes('textrad.in/app');
    
    console.log(`Tab ${tabId} is on: ${isMainSite ? 'Main site' : (isAppPage ? 'App page' : 'Other TextRad page')}`);
    
    // Look for cookies first - most reliable across both sites
    try {
      const cookieCheckResults = await chrome.scripting.executeScript({
        target: { tabId: tabId },
        func: () => {
          try {
            // Check for cookies that indicate login
            const cookies = document.cookie.split('; ');
            for (const cookie of cookies) {
              const [name, value] = cookie.split('=');
              if ((name.includes('auth') || name.includes('session') || name.includes('token') || name.includes('user')) && 
                  value && value.length > 10) {
                return { loggedIn: true, reason: `Found auth cookie: ${name}` };
              }
            }
            return null; // No conclusive result from cookies
          } catch (e) {
            console.error('Error checking cookies:', e);
            return null;
          }
        }
      });
      
      if (cookieCheckResults && cookieCheckResults[0] && cookieCheckResults[0].result) {
        console.log(`Cookie check for tab ${tabId}: ${cookieCheckResults[0].result.loggedIn} (${cookieCheckResults[0].result.reason})`);
        return cookieCheckResults[0].result.loggedIn;
      }
    } catch (error) {
      console.log(`Error checking cookies in tab ${tabId}:`, error);
    }
    
    // Main site specific login check with more aggressive DOM element checks
    if (isMainSite) {
      try {
        const mainSiteCheckResults = await chrome.scripting.executeScript({
          target: { tabId: tabId },
          func: () => {
            // Check for main site specific login indicators
            // First check for localStorage items
            try {
              // Check for firebase auth in localStorage (most reliable)
              const firebaseAuth = localStorage.getItem('firebase:authUser:AIzaSyBiQwFjWCjJA1vXANQ2igNMlkTwAk8CJzA:[DEFAULT]');
              if (firebaseAuth && firebaseAuth.length > 20) {
                return { loggedIn: true, reason: 'Found Firebase auth in localStorage' };
              }
              
              // Main site specific auth items
              const mainSiteAuthItems = [
                'firebaseui:rememberedAccounts',
                'firebase:authUser',
                'auth',
                'authState',
                'authUser',
                'textrad_user',
                'user_data',
                'session',
                'sessionData'
              ];
              
              for (const key of mainSiteAuthItems) {
                const item = localStorage.getItem(key);
                if (item && item.length > 10) {
                  return { loggedIn: true, reason: `Found auth item in localStorage: ${key}` };
                }
              }
            } catch (e) {
              console.log('Error checking localStorage:', e);
            }
            
            // Check for login/logout UI elements that would only be visible for logged-in users
            try {
              // For main site, check for account menu, profile menu, or dashboard elements
              const mainSiteLoggedInSelectors = [
                // Navigation and menu elements when logged in
                '.account-menu', '.user-menu', '.profile-menu', '.dashboard-menu',
                // User display elements
                '.user-avatar', '.user-profile', '.user-initials', '.user-greeting',
                // Common dashboard elements
                '.dashboard', '.dashboard-panel', '.dashboard-content',
                // Account-related sections
                '.my-account', '.my-reports', '.my-dashboard', '.account-section',
                // Action buttons only visible when logged in
                '#logout-button', '.logout-button', '.sign-out-button',
                // App link that only appears when logged in
                '.app-link', '[href*="/app"]'
              ];
              
              // Look for these elements and check if they're visible
              for (const selector of mainSiteLoggedInSelectors) {
                const elements = document.querySelectorAll(selector);
                if (elements && elements.length > 0) {
                  for (const element of elements) {
                    // Check if element is visible and has content
                    if (window.getComputedStyle(element).display !== 'none' &&
                        element.offsetWidth > 0 && element.offsetHeight > 0) {
                      return { loggedIn: true, reason: `Found main site logged-in element: ${selector}` };
                    }
                  }
                }
              }
              
              // Check for login links/buttons which would only be visible when NOT logged in
              const mainSiteLoggedOutSelectors = [
                '.login-button', '.sign-in-button', '.auth-button',
                '#login-button', '#sign-in-button',
                '[href*="login"]', '[href*="signin"]', '[href*="auth"]'
              ];
              
              for (const selector of mainSiteLoggedOutSelectors) {
                const elements = document.querySelectorAll(selector);
                if (elements && elements.length > 0) {
                  for (const element of elements) {
                    // Check if the login element is visible
                    if (window.getComputedStyle(element).display !== 'none' &&
                        element.offsetWidth > 0 && element.offsetHeight > 0) {
                      return { loggedIn: false, reason: `Found main site login element: ${selector}` };
                    }
                  }
                }
              }
              
              // If we have the App button/link showing prominently, user is likely logged in
              const appLinks = Array.from(document.querySelectorAll('a')).filter(a => 
                (a.textContent.includes('App') || a.textContent.includes('Dashboard') || a.textContent.includes('Go to App')) &&
                a.href.includes('/app')
              );
              
              if (appLinks.length > 0) {
                for (const link of appLinks) {
                  if (window.getComputedStyle(link).display !== 'none' &&
                      link.offsetWidth > 0 && link.offsetHeight > 0) {
                    return { loggedIn: true, reason: 'Found visible App link' };
                  }
                }
              }
              
              // Check account/profile related links
              const accountLinks = Array.from(document.querySelectorAll('a')).filter(a => 
                a.textContent.includes('Account') || 
                a.textContent.includes('Profile') || 
                a.textContent.includes('Dashboard') ||
                a.textContent.includes('Reports') ||
                a.textContent.includes('Logout') ||
                a.textContent.includes('Sign Out')
              );
              
              if (accountLinks.length > 0) {
                for (const link of accountLinks) {
                  if (window.getComputedStyle(link).display !== 'none' &&
                      link.offsetWidth > 0 && link.offsetHeight > 0) {
                    return { loggedIn: true, reason: 'Found account-related links' };
                  }
                }
              }
            } catch (e) {
              console.log('Error checking DOM elements:', e);
            }
            
            // No conclusive indicators found
            return null;
          }
        });
        
        if (mainSiteCheckResults && mainSiteCheckResults[0] && mainSiteCheckResults[0].result) {
          console.log(`Main site check for tab ${tabId}: ${mainSiteCheckResults[0].result.loggedIn} (${mainSiteCheckResults[0].result.reason})`);
          return mainSiteCheckResults[0].result.loggedIn;
        }
      } catch (error) {
        console.log(`Error performing main site check in tab ${tabId}:`, error);
      }
    }
    
    // Standard checks for both sites
    try {
      // This script will run directly in the page context and check for login indicators
      const checkResults = await chrome.scripting.executeScript({
        target: { tabId: tabId },
        func: () => {
          // STORAGE CHECKS (most reliable)
          // Check localStorage for auth tokens or user data
          try {
            // Look for common auth tokens in localStorage
            const authItems = [
              'firebase:authUser',
              'authUser',
              'textrad_user',
              'user_data',
              'auth_token',
              'session_token',
              'user'
            ];
            
            for (const key of authItems) {
              const item = localStorage.getItem(key);
              if (item && item.length > 10) { // Ensure it's not empty
                return { loggedIn: true, reason: `Found auth token in localStorage: ${key}` };
              }
            }
            
            // Also check if any localStorage key contains auth-related terms
            for (let i = 0; i < localStorage.length; i++) {
              const key = localStorage.key(i);
              if (key && (
                  key.includes('auth') || 
                  key.includes('user') || 
                  key.includes('token') ||
                  key.includes('session')
                )) {
                const value = localStorage.getItem(key);
                if (value && value.length > 10) {
                  return { loggedIn: true, reason: `Found auth-related localStorage item: ${key}` };
                }
              }
            }
          } catch (e) {
            console.log('Error checking localStorage:', e);
          }
          
          // WINDOW OBJECT CHECKS
          // Check for user data in global window object
          try {
            if (window.firebase && window.firebase.auth && window.firebase.auth().currentUser) {
              return { loggedIn: true, reason: 'Found firebase auth user' };
            }
            
            if (window.textradApp && window.textradApp.user) {
              return { loggedIn: true, reason: 'Found textradApp.user' };
            }
            
            if (window.textradApp && window.textradApp.isAuthenticated) {
              return { loggedIn: true, reason: 'Found textradApp.isAuthenticated = true' };
            }
            
            if (window.textradApp && window.textradApp.currentUser) {
              return { loggedIn: true, reason: 'Found textradApp.currentUser' };
            }
            
            if (window.USER_DATA || window.user || window.currentUser) {
              return { loggedIn: true, reason: 'Found user data in window object' };
            }
          } catch (e) {
            console.log('Error checking window objects:', e);
          }
          
          // DOM ELEMENT CHECKS
          // Check for user UI elements that would only be visible when logged in
          const userProfileElements = [
            '#user-display-name',
            '#user-initials',
            '#user-avatar',
            '#profile-email',
            '#logout-button',
            '.user-avatar',
            '.user-profile',
            '.user-menu'
          ];
          
          for (const selector of userProfileElements) {
            const elements = document.querySelectorAll(selector);
            if (elements && elements.length > 0) {
              for (const element of elements) {
                if ((element.innerText && element.innerText.trim().length > 0) ||
                    window.getComputedStyle(element).display !== 'none') {
                  return { loggedIn: true, reason: `Found user element ${selector}` };
                }
              }
            }
          }
          
          // Check for app-specific elements that would only be present for logged in users
          const appSpecificElements = [
            '#reports-section',
            '#recorder-section',
            '#report-section',
            '.reports-list-container',
            '#back-to-reports',
            '.my-account',
            '.my-reports',
            '.dashboard',
            '.account-section'
          ];
          
          for (const selector of appSpecificElements) {
            const elements = document.querySelectorAll(selector);
            if (elements && elements.length > 0) {
              for (const element of elements) {
                if (window.getComputedStyle(element).display !== 'none') {
                  return { loggedIn: true, reason: `Found app element ${selector}` };
                }
              }
            }
          }
          
          // Check for login page elements that would only be visible when not logged in
          const loginElements = [
            '#firebaseui-auth-container',
            '.firebaseui-container',
            '.login-container',
            '.login-form',
            '.sign-in-form',
            '.sign-in-button:not(.hidden)',
            '#login-button:not(.hidden)',
            '#sign-in-button:not(.hidden)'
          ];
          
          for (const selector of loginElements) {
            const elements = document.querySelectorAll(selector);
            if (elements && elements.length > 0) {
              for (const element of elements) {
                if (window.getComputedStyle(element).display !== 'none') {
                  return { loggedIn: false, reason: `Found login element ${selector}` };
                }
              }
            }
          }
          
          // Check for links that would indicate a logged-in state
          const accountLinks = Array.from(document.querySelectorAll('a')).filter(a => 
            a.textContent.includes('Account') || 
            a.textContent.includes('Profile') || 
            a.textContent.includes('Dashboard') ||
            a.textContent.includes('Reports') ||
            a.textContent.includes('Logout') ||
            a.textContent.includes('Sign Out')
          );
          
          if (accountLinks.length > 0) {
            return { loggedIn: true, reason: 'Found account-related links' };
          }
          
          // Default: Assume not logged in for safety
          return { loggedIn: false, reason: 'No login indicators found' };
        }
      });
      
      // Check if we got a result
      if (checkResults && checkResults[0] && typeof checkResults[0].result?.loggedIn === 'boolean') {
        const result = checkResults[0].result;
        console.log(`Standard login check for tab ${tabId}: ${result.loggedIn} (${result.reason})`);
        return result.loggedIn;
      }
    } catch (error) {
      console.log(`Error executing login check script in tab ${tabId}:`, error);
      // Continue to try content script approach
    }
    
    // Now try the content script approach as a backup
    try {
      const response = await chrome.tabs.sendMessage(tabId, { action: 'CHECK_LOGIN' });
      if (response && typeof response.loggedIn === 'boolean') {
        console.log(`Content script login check for tab ${tabId}: ${response.loggedIn}`);
        return response.loggedIn;
      }
    } catch (error) {
      // Content script might not be loaded, try to inject it
      console.log(`Content script not loaded in tab ${tabId}, injecting it`);
      
      try {
        await chrome.scripting.executeScript({
          target: { tabId: tabId },
          files: ['content/content-script.js']
        });
        
        // Wait a moment for script to initialize
        await new Promise(resolve => setTimeout(resolve, 500));
        
        // Try again
        try {
          const retryResponse = await chrome.tabs.sendMessage(tabId, { action: 'CHECK_LOGIN' });
          if (retryResponse && typeof retryResponse.loggedIn === 'boolean') {
            console.log(`Injected content script login check for tab ${tabId}: ${retryResponse.loggedIn}`);
            return retryResponse.loggedIn;
          }
        } catch (retryError) {
          console.log(`Still failed to check login state after injection: ${retryError.message}`);
          // Don't return false here, we'll use the executeScript result if available
        }
      } catch (injectionError) {
        console.log(`Failed to inject content script in tab ${tabId}:`, injectionError);
      }
    }
    
    // If we get here, something went wrong with all approaches
    // Let's check specifically for main site login as a last resort if that's where we are
    if (isMainSite) {
      try {
        // One final check - if there's a visible link to /app, the user is probably logged in
        const appLinkCheck = await chrome.scripting.executeScript({
          target: { tabId: tabId },
          func: () => {
            const appLinks = Array.from(document.querySelectorAll('a[href*="/app"]')).filter(a => 
              window.getComputedStyle(a).display !== 'none' && a.offsetWidth > 0 && a.offsetHeight > 0
            );
            return appLinks.length > 0;
          }
        });
        
        if (appLinkCheck && appLinkCheck[0] && appLinkCheck[0].result === true) {
          console.log(`Found visible app link in tab ${tabId}, assuming logged in`);
          return true;
        }
      } catch (error) {
        console.log(`Error performing final app link check in tab ${tabId}:`, error);
      }
    }
    
    console.log(`All login detection methods failed for tab ${tabId}, assuming not logged in`);
    return false;
  } catch (error) {
    console.error(`Error checking login state for tab ${tabId}:`, error);
    return false;
  }
}

/**
 * Save state to storage
 */
function saveState() {
  chrome.storage.local.set({
    textradTabs: textradTabs,
    currentState: currentState
  });
}

// Initialize service worker
initialize();