/**
 * TextRad Dictation Extension - Content Script
 * 
 * Responsibilities:
 * - Inject into TextRad web app
 * - Communicate with extension background script
 * - Interact with TextRad web app
 * - Inject dictation data into TextRad UI
 */

// Track elements and state
let transcriptionElement = null;
let formatSelectorElement = null;
let currentFormat = null;
let isConnected = false;

/**
 * Initialize content script
 */
function initialize() {
  console.log('TextRad Dictation Extension content script initialized');
  
  // Check if this is a TextRad page
  if (!isTextRadPage()) {
    console.log('Not a TextRad page, content script will not run');
    return;
  }
  
  // Log page info
  console.log('TextRad page detected:', window.location.href);
  
  // Wait for the page to be fully loaded
  if (document.readyState === 'complete') {
    initializeOnTextRadPage();
  } else {
    window.addEventListener('load', initializeOnTextRadPage);
  }
}

/**
 * Start continuous monitoring of the generate button
 * This ensures the button stays visible after format selection
 */
let generateButtonMonitoringInterval = null;
function startGenerateButtonMonitoring() {
  // Clear any existing interval
  if (generateButtonMonitoringInterval) {
    clearInterval(generateButtonMonitoringInterval);
  }
  
  // Set a new interval that regularly checks and enforces visibility
  generateButtonMonitoringInterval = setInterval(() => {
    // Check if we have the conditions for showing the generate button
    const transcriptionText = document.getElementById('transcription-text');
    const formatSelector = document.getElementById('format-selector');
    
    // If we have both transcription content and a selected format
    if (transcriptionText && 
        transcriptionText.value && 
        transcriptionText.value.trim().length > 0 &&
        formatSelector && 
        formatSelector.value && 
        formatSelector.value !== "") {
      
      // Check the visibility of the generate button
      const generateContainer = document.querySelector('#generate-container');
      const generateButton = document.querySelector('#generate-button');
      
      // If either the container or button exists and is hidden/not visible, make it visible
      if ((generateContainer && 
          (generateContainer.classList.contains('hidden') || 
           generateContainer.style.display === 'none')) ||
         (generateButton && 
          (generateButton.classList.contains('hidden') || 
           generateButton.style.display === 'none'))) {
        
        console.log('Generate button monitor: Button should be visible but is hidden, fixing...');
        ensureGenerateButtonVisible();
      }
    }
  }, 500); // Check every 500ms to avoid too much CPU usage
  
  console.log('Generate button continuous monitoring started');
}

/**
 * Initialize on confirmed TextRad page
 */
function initializeOnTextRadPage() {
  // Inject script to help with login detection
  injectLoginDetectionScript();
  
  // Connect to background script
  connectToBackground();
  
  // Set up listeners
  setupListeners();
  
  // Find key elements
  findUIElements();
  
  // Try again in 2 seconds if elements weren't found
  if (!transcriptionElement) {
    setTimeout(findUIElements, 2000);
  }
  
  // Initialize injection bridge
  initializeInjectionBridge();
}

/**
 * Inject a script to help with detecting login state
 * This runs in page context and can access window variables directly
 */
function injectLoginDetectionScript() {
  const script = document.createElement('script');
  script.textContent = `
    // TextRad Login Detection Script
    (function() {
      console.log('TextRad Login Detection Script initialized');
      
      // Function to check login state
      function checkLoginState() {
        let isLoggedIn = false;
        let reason = 'unknown';
        
        // Detect if we're on main site or app
        const isMainSite = window.location.pathname === '/' || 
                         window.location.pathname.includes('/index') || 
                         window.location.pathname === '';
                         
        const isAppPage = window.location.pathname.includes('/app');
        
        console.log('Checking login state on ' + (isMainSite ? 'main site' : (isAppPage ? 'app page' : 'other page')));
        
        // Check for all possible auth indicators
        
        // 1. Check Firebase auth - most reliable when available
        if (window.firebase && window.firebase.auth && window.firebase.auth().currentUser) {
          isLoggedIn = true;
          reason = 'firebase auth detected';
        } 
        // 2. Check TextRad app state - for app page
        else if (window.textradApp && window.textradApp.user) {
          isLoggedIn = true;
          reason = 'textradApp.user detected';
        }
        // 3. Check TextRad app authentication state
        else if (window.textradApp && window.textradApp.isAuthenticated === true) {
          isLoggedIn = true;
          reason = 'textradApp.isAuthenticated = true';
        }
        // 4. Check TextRad app current user
        else if (window.textradApp && window.textradApp.currentUser) {
          isLoggedIn = true;
          reason = 'textradApp.currentUser detected';
        }
        // 5. Check other app variables
        else if (window.USER_DATA || window.user || window.currentUser) {
          isLoggedIn = true;
          reason = 'global user data object detected';
        }
        // 6. Check for Firebase auth in localStorage
        else if (localStorage.getItem('firebase:authUser:AIzaSyBiQwFjWCjJA1vXANQ2igNMlkTwAk8CJzA:[DEFAULT]')) {
          isLoggedIn = true;
          reason = 'firebase auth in localStorage';
        }
        // 7. Check other localStorage auth items
        else if (localStorage.getItem('authUser') || 
                localStorage.getItem('textrad_user') || 
                localStorage.getItem('user_data')) {
          isLoggedIn = true;
          reason = 'auth data in localStorage';
        }
        // 8. Check cookies for auth tokens
        else {
          try {
            const cookies = document.cookie.split('; ');
            for (const cookie of cookies) {
              const [name, value] = cookie.split('=');
              if ((name.includes('auth') || name.includes('session') || name.includes('token') || name.includes('user')) && 
                  value && value.length > 10) {
                isLoggedIn = true;
                reason = 'auth cookie detected: ' + name;
                break;
              }
            }
          } catch (e) {
            console.error('Error checking cookies:', e);
          }
        }
        
        // If still not determined, check DOM elements
        if (!isLoggedIn) {
          // Check for login/logout UI elements on main site
          if (isMainSite) {
            // Check for elements visible when logged in on main site
            const mainSiteLoggedInSelectors = [
              // Navigation and menu elements when logged in
              '.account-menu', '.user-menu', '.profile-menu', '.dashboard-menu',
              // User display elements
              '.user-avatar', '.user-profile', '.user-initials', '.user-greeting',
              // Common dashboard elements
              '.dashboard', '.dashboard-panel', '.dashboard-content',
              // Account-related sections
              '.my-account', '.my-reports', '.my-dashboard', '.account-section',
              // Action buttons only visible when logged in
              '#logout-button', '.logout-button', '.sign-out-button'
            ];
            
            for (const selector of mainSiteLoggedInSelectors) {
              const elements = document.querySelectorAll(selector);
              if (elements && elements.length > 0) {
                for (const element of elements) {
                  // Check if element is visible and has content
                  if (window.getComputedStyle(element).display !== 'none' &&
                      element.offsetWidth > 0 && element.offsetHeight > 0) {
                    isLoggedIn = true;
                    reason = 'main site logged-in element: ' + selector;
                    break;
                  }
                }
                if (isLoggedIn) break;
              }
            }
            
            // If not determined yet, check for visible links to /app page
            // On main site, a link to the app usually means the user is logged in
            if (!isLoggedIn) {
              const appLinks = Array.from(document.querySelectorAll('a[href*="/app"]')).filter(a => 
                window.getComputedStyle(a).display !== 'none' && a.offsetWidth > 0 && a.offsetHeight > 0
              );
              
              if (appLinks.length > 0) {
                isLoggedIn = true;
                reason = 'visible app link on main site';
              }
            }
            
            // If not determined yet, check for elements visible when NOT logged in
            if (!isLoggedIn) {
              const mainSiteLoggedOutSelectors = [
                '.login-button', '.sign-in-button', '.auth-button',
                '#login-button', '#sign-in-button',
                '[href*="login"]', '[href*="signin"]', '[href*="auth"]'
              ];
              
              let foundLoggedOutElement = false;
              for (const selector of mainSiteLoggedOutSelectors) {
                const elements = document.querySelectorAll(selector);
                if (elements && elements.length > 0) {
                  for (const element of elements) {
                    // Check if the login element is visible
                    if (window.getComputedStyle(element).display !== 'none' &&
                        element.offsetWidth > 0 && element.offsetHeight > 0) {
                      foundLoggedOutElement = true;
                      reason = 'visible login element: ' + selector;
                      break;
                    }
                  }
                  if (foundLoggedOutElement) break;
                }
              }
              
              // If we found logged-out element, we are NOT logged in
              if (foundLoggedOutElement) {
                isLoggedIn = false;
              }
            }
          }
          // For app page, check app-specific elements
          else if (isAppPage) {
            const appPageElements = [
              '#reports-section',
              '#recorder-section',
              '#transcription-container',
              '#format-selector',
              '.reports-list-container',
              '#back-to-reports',
              '#user-display-name',
              '#user-initials',
              '#user-avatar',
              '#profile-email',
              '#logout-button'
            ];
            
            for (const selector of appPageElements) {
              const elements = document.querySelectorAll(selector);
              if (elements && elements.length > 0) {
                for (const element of elements) {
                  if (window.getComputedStyle(element).display !== 'none') {
                    isLoggedIn = true;
                    reason = 'app element visible: ' + selector;
                    break;
                  }
                }
                if (isLoggedIn) break;
              }
            }
          }
        }
        
        // Dispatch result to content script
        document.dispatchEvent(new CustomEvent('textrad-login-state', {
          detail: { 
            isLoggedIn: isLoggedIn,
            reason: reason,
            isMainSite: isMainSite,
            isAppPage: isAppPage,
            timestamp: Date.now()
          }
        }));
        
        return isLoggedIn;
      }
      
      // Check immediately
      checkLoginState();
      
      // Set up interval to check periodically
      setInterval(checkLoginState, 10000);
      
      // Expose for extension to call
      window.checkTextRadLoginState = checkLoginState;
    })();
  `;
  
  // Append to document
  (document.head || document.documentElement).appendChild(script);
  
  // Clean up
  script.remove();
  
  // Listen for login state events from the injected script
  document.addEventListener('textrad-login-state', (event) => {
    const { isLoggedIn, reason, isMainSite, isAppPage } = event.detail;
    console.log(`Login state from injected script: ${isLoggedIn} (${reason}) on ${isMainSite ? 'main site' : (isAppPage ? 'app page' : 'other page')}`);
    
    // Store this information for the content script to use
    window.detectedLoginState = {
      isLoggedIn: isLoggedIn,
      reason: reason,
      isMainSite: isMainSite,
      isAppPage: isAppPage,
      timestamp: Date.now()
    };
  });
}

/**
 * Check if current page is a TextRad page and user is logged in
 */
function isTextRadPage() {
  // First check if we're on a TextRad domain
  const isTextRadDomain = window.location.hostname.includes('textrad.com') || 
                         window.location.hostname.includes('textrad.in') || 
                         window.location.hostname.includes('localhost');
  
  if (!isTextRadDomain) {
    return false;
  }
  
  // Check if user is logged in
  const isLoggedIn = checkIfUserIsLoggedIn();
  
  // For debugging - print the full URL to see if we're on main site or app page
  const isMainSite = window.location.pathname === '/' || 
                     window.location.pathname.includes('/index') || 
                     window.location.pathname === '';
                     
  const isAppPage = window.location.pathname.includes('/app');
  
  console.log(`TextRad page check - URL: ${window.location.href}, Domain: ${isTextRadDomain}, ` +
              `Main site: ${isMainSite}, App page: ${isAppPage}, Logged in: ${isLoggedIn}`);
  
  if (!isLoggedIn) {
    console.log('User not logged in to TextRad, not considering as active TextRad page');
    return false;
  }
  
  return true;
}

/**
 * Check if user is logged in to TextRad
 */
function checkIfUserIsLoggedIn() {
  // First, check our injected script results which is the most reliable
  if (window.detectedLoginState && window.detectedLoginState.isLoggedIn) {
    console.log(`Injected script detected logged in state: ${window.detectedLoginState.reason}`);
    return true;
  }
  
  // Try to call the window function if available
  try {
    if (window.checkTextRadLoginState && typeof window.checkTextRadLoginState === 'function') {
      const isLoggedIn = window.checkTextRadLoginState();
      if (isLoggedIn) {
        console.log('Login check via window function successful');
        return true;
      }
    }
  } catch (error) {
    console.log('Error calling window login check function:', error);
  }
  
  // Check for Firebase auth directly
  if (window.firebase && window.firebase.auth && window.firebase.auth().currentUser) {
    console.log('Firebase auth indicates user is logged in');
    return true;
  }
  
  // Check for authentication elements on the main page
  const authElements = [
    // User menu and profile elements (both main site and app)
    '.user-menu',
    '.user-profile',
    '.profile-menu',
    '.account-menu',
    '.user-avatar',
    '#user-avatar',
    '#user-profile-menu',
    '#user-menu',
    '#user-display-name',
    '#user-initials',
    '#logout-button',
    '#profile-email',
    
    // Dashboard elements that indicate logged-in state
    '.dashboard-content',
    '.account-dashboard',
    '.reports-dashboard',
    '.user-reports',
    
    // App-specific elements 
    '#reports-section:not(.hidden)',
    '#recorder-section:not(.hidden)',
    '#report-section:not(.hidden)',
    '.reports-list-container:not(.hidden)',
    '#back-to-reports',
    
    // Main site elements for logged-in users
    '.my-account',
    '.my-reports',
    '.subscription-details',
    '.welcome-message'
  ];
  
  for (const selector of authElements) {
    const elements = document.querySelectorAll(selector);
    if (elements && elements.length > 0) {
      for (const element of elements) {
        // Check if element has content or is visible
        if (
          (element.innerText && element.innerText.trim().length > 0) ||
          (element.textContent && element.textContent.trim().length > 0) ||
          element.classList.contains('logged-in') ||
          window.getComputedStyle(element).display !== 'none'
        ) {
          console.log(`Found logged-in indicator: ${selector}`);
          return true;
        }
      }
    }
  }
  
  // Check for login form elements - if these are visible, user is NOT logged in
  const loginFormElements = [
    '#firebaseui-auth-container',
    '.firebaseui-container',
    '.login-container',
    '.login-form',
    '.sign-in-form',
    '.sign-in-button:not(.hidden)',
    '#login-button:not(.hidden)',
    '#sign-in-button:not(.hidden)',
    '.auth-form'
  ];
  
  for (const selector of loginFormElements) {
    const elements = document.querySelectorAll(selector);
    if (elements && elements.length > 0) {
      for (const element of elements) {
        if (window.getComputedStyle(element).display !== 'none') {
          console.log(`Found login form element, user appears to be logged out: ${selector}`);
          return false;
        }
      }
    }
  }
  
  // On main page, check for special dashboard elements
  if (window.location.pathname === '/' || window.location.pathname === '/index.html') {
    // Check for dashboard elements
    const dashboardElements = [
      '.dashboard',
      '.dashboard-panel', 
      '.reports-panel',
      '.user-welcome',
      '.account-section'
    ];
    
    for (const selector of dashboardElements) {
      if (document.querySelector(selector)) {
        console.log(`Found dashboard element on main page: ${selector}, user appears logged in`);
        return true;
      }
    }
    
    // Check for "My Account" or similar links
    const accountLinks = Array.from(document.querySelectorAll('a')).filter(a => 
      a.textContent.includes('Account') || 
      a.textContent.includes('Profile') || 
      a.textContent.includes('Dashboard') ||
      a.textContent.includes('Reports')
    );
    
    if (accountLinks.length > 0) {
      console.log('Found account-related links, user appears logged in');
      return true;
    }
  }
  
  // On app page, check if the format selector is present
  if (window.location.pathname.includes('/app')) {
    const appElements = [
      '#format-selector',
      '#transcription-container',
      '#recorder-section',
      '#reports-section'
    ];
    
    for (const selector of appElements) {
      if (document.querySelector(selector)) {
        console.log(`Found app element: ${selector}, user appears logged in`);
        return true;
      }
    }
  }
  
  // Check cookies for authentication token
  if (document.cookie.includes('auth') || 
      document.cookie.includes('token') || 
      document.cookie.includes('session')) {
    console.log('Found authentication cookie, user may be logged in');
    return true;
  }
  
  // Default: If we can't definitively determine login state, assume not logged in for safety
  console.log('Could not determine login state, assuming user is not logged in');
  return false;
}

/**
 * Connect to background script
 */
function connectToBackground() {
  chrome.runtime.sendMessage({ action: 'CONNECTED_TO_TEXTRAD' }, (response) => {
    if (chrome.runtime.lastError) {
      console.error('Error connecting to background:', chrome.runtime.lastError);
      return;
    }
    
    if (response && response.success) {
      isConnected = true;
      console.log('Connected to TextRad Extension background script');
    }
  });
}

/**
 * Set up message and event listeners
 */
function setupListeners() {
  // Listen for messages from background script
  chrome.runtime.onMessage.addListener(handleBackgroundMessage);
  
  // Listen for custom events from the web app
  window.addEventListener('textrad-webapp-message', handleWebAppMessage);
  
  // Listen for DOM mutations to detect UI changes
  setupMutationObserver();
}

/**
 * Handle messages from background script
 */
function handleBackgroundMessage(message, sender, sendResponse) {
  console.log('Received message from background:', message);
  
  // Handle different message types
  switch (message.action) {
    case 'PING':
      // Simple ping to check if content script is loaded
      console.log('Received ping from background script');
      // Also check login state and include it in the response
      const isLoggedIn = checkIfUserIsLoggedIn();
      sendResponse({ 
        success: true,
        loggedIn: isLoggedIn
      });
      break;
      
    case 'CHECK_LOGIN':
      // Specifically check if the user is logged in
      const loginStatus = checkIfUserIsLoggedIn();
      console.log('Login check result:', loginStatus);
      sendResponse({ 
        loggedIn: loginStatus
      });
      break;
      
    case 'INJECT_TRANSCRIPTION':
      // Check login state before allowing injection
      if (!checkIfUserIsLoggedIn()) {
        console.error('Cannot inject transcription: User not logged in to TextRad');
        sendResponse({ 
          success: false, 
          error: 'User not logged in to TextRad. Please log in and go to TextRad Web App to use the plugin.'
        });
        break;
      }
      
      const success = injectTranscription(message.transcription);
      sendResponse({ success: success });
      break;
      
    case 'GET_TEXTRAD_STATE':
      const state = getTextRadState();
      // Include login state in the response
      state.loggedIn = checkIfUserIsLoggedIn();
      sendResponse({ success: true, state: state });
      break;
  }
  
  // Return true to indicate async response
  return true;
}

/**
 * Handle custom events from the web app
 */
function handleWebAppMessage(event) {
  const { type, payload } = event.detail;
  
  console.log('Received message from web app:', type, payload);
  
  // Forward message to background script
  chrome.runtime.sendMessage({
    action: 'WEBAPP_MESSAGE',
    type: type,
    payload: payload
  });
}

/**
 * Set up mutation observer to detect UI changes
 */
function setupMutationObserver() {
  const observer = new MutationObserver((mutations) => {
    // Check for relevant UI changes
    let needsUIRefresh = false;
    
    for (const mutation of mutations) {
      if (mutation.type === 'childList' || mutation.type === 'attributes') {
        needsUIRefresh = true;
        break;
      }
    }
    
    if (needsUIRefresh) {
      // Refresh UI elements
      findUIElements();
    }
  });
  
  // Start observing
  observer.observe(document.body, {
    childList: true,
    subtree: true,
    attributes: true,
    attributeFilter: ['class', 'style']
  });
}

/**
 * Find important UI elements in the TextRad app
 */
function findUIElements() {
  console.log('Finding UI elements in TextRad app');
  
  // Start with known selectors for the TextRad app
  // Try to find the transcription container first using various selectors
  let transcriptionContainer = document.querySelector('.transcription-container');
  if (!transcriptionContainer) {
    // Try alternative selectors in order of likelihood
    const alternativeContainers = [
      '.recorder-container',
      '.dictation-container',
      '.editor-container',
      '.text-input-container',
      '.report-container'
    ];
    
    for (const selector of alternativeContainers) {
      transcriptionContainer = document.querySelector(selector);
      if (transcriptionContainer) {
        console.log(`Found container using selector: ${selector}`);
        break;
      }
    }
  }
  
  // If we found a container, look for input elements within it
  if (transcriptionContainer) {
    console.log('Found transcription container:', transcriptionContainer);
    
    // Try specific ID first (most reliable)
    transcriptionElement = document.querySelector('#transcription-text');
    
    if (!transcriptionElement) {
      // Look for common TextRad selectors
      const textradSelectors = [
        '#dictation-text',
        '#report-text',
        '#editor-text',
        '.transcription-input',
        '.report-text-area'
      ];
      
      for (const selector of textradSelectors) {
        transcriptionElement = document.querySelector(selector);
        if (transcriptionElement) {
          console.log(`Found transcription element using selector: ${selector}`);
          break;
        }
      }
      
      // If still not found, look for any textarea or contenteditable div within the container
      if (!transcriptionElement) {
        transcriptionElement = transcriptionContainer.querySelector('textarea, [contenteditable="true"], .ProseMirror, .editor, .quill-editor');
        console.log('Looked for textarea/contenteditable/editor:', transcriptionElement);
      }
    }
    
    // If still not found, look more broadly
    if (!transcriptionElement) {
      // Look for any textarea or rich text editor on the page
      const possibleEditors = [
        'textarea', 
        '[contenteditable="true"]', 
        '.ProseMirror', 
        '.ql-editor',
        '.CodeMirror',
        '.editor-instance',
        '.monaco-editor'
      ];
      
      for (const selector of possibleEditors) {
        transcriptionElement = document.querySelector(selector);
        if (transcriptionElement) {
          console.log(`Found editor using selector: ${selector}`);
          break;
        }
      }
    }
    
    console.log('Found transcription element:', transcriptionElement);
  } else {
    console.log('No transcription container found, looking for standalone editor elements');
    
    // If we couldn't find a container, try directly finding editor elements
    const editorSelectors = [
      '#transcription-text',
      '#dictation-text',
      '#report-text',
      'textarea',
      '[contenteditable="true"]',
      '.ProseMirror',
      '.ql-editor',
      '.editor-instance',
      '.CodeMirror',
      '.monaco-editor'
    ];
    
    for (const selector of editorSelectors) {
      transcriptionElement = document.querySelector(selector);
      if (transcriptionElement) {
        console.log(`Found standalone editor using selector: ${selector}`);
        break;
      }
    }
  }
  
  // Find format selector - try multiple selectors
  formatSelectorElement = document.querySelector('#format-selector');
  if (!formatSelectorElement) {
    // Try alternative selectors
    const formatSelectors = [
      'select.format-selector',
      'select.report-format',
      'select.template-selector',
      'select:not([disabled])'
    ];
    
    for (const selector of formatSelectors) {
      formatSelectorElement = document.querySelector(selector);
      if (formatSelectorElement) {
        console.log(`Found format selector using selector: ${selector}`);
        break;
      }
    }
  }
  
  if (formatSelectorElement) {
    console.log('Found format selector:', formatSelectorElement);
    
    // Get selected format
    currentFormat = {
      id: formatSelectorElement.value,
      name: formatSelectorElement.options[formatSelectorElement.selectedIndex]?.text || ''
    };
    
    // Listen for format changes
    formatSelectorElement.addEventListener('change', handleFormatChange);
    
    console.log('Current format:', currentFormat);
  } else {
    console.log('No format selector found');
  }
  
  // Log overall findings
  console.log('UI element detection results:', {
    foundTranscriptionElement: !!transcriptionElement,
    foundFormatSelector: !!formatSelectorElement
  });
}

/**
 * Handle format selection changes
 */
function handleFormatChange(event) {
  currentFormat = {
    id: event.target.value,
    name: event.target.options[event.target.selectedIndex]?.text || ''
  };
  
  // Notify background script of format change
  chrome.runtime.sendMessage({
    action: 'FORMAT_CHANGED',
    format: currentFormat
  });
  
  // Ensure the generate button is visible after format selection
  // This is crucial as the web app often hides it after format selection
  console.log('Format changed to:', currentFormat.name);
  setTimeout(ensureGenerateButtonVisible, 100);
  setTimeout(ensureGenerateButtonVisible, 500);
  setTimeout(ensureGenerateButtonVisible, 1000);
}

/**
 * Get current TextRad state
 */
function getTextRadState() {
  return {
    hasTranscriptionElement: !!transcriptionElement,
    hasFormatSelector: !!formatSelectorElement,
    currentFormat: currentFormat,
    url: window.location.href,
    title: document.title
  };
}

/**
 * Initialize injection bridge for communication with TextRad app
 */
function initializeInjectionBridge() {
  // Create and inject a bridge script element
  const script = document.createElement('script');
  script.textContent = `
    // TextRad Extension Bridge
    (function() {
      console.log('TextRad Extension Bridge initialized');
      
      // Set up event listeners for web app to extension communication
      document.addEventListener('textrad-extension-message', (event) => {
        console.log('Bridge received message for extension:', event.detail);
      });
      
      // Expose method to send messages from web app to extension
      window.sendToTextRadExtension = function(type, payload) {
        document.dispatchEvent(new CustomEvent('textrad-webapp-message', {
          detail: { type, payload }
        }));
      };
      
      // Store original app methods
      let originalHandleFormatSelection = null;
      let originalShowGenerateButton = null;
      let originalHideGenerateButton = null;
      let originalUpdateUI = null;
      
      // Monitor the TextRad app object and override methods as soon as they're available
      function setupAppOverrides() {
        if (window.textradApp) {
          console.log('Found textradApp object, overriding methods');
          
          // Store original app functions if they exist
          originalShowGenerateButton = window.textradApp.showGenerateButton;
          originalHideGenerateButton = window.textradApp.hideGenerateButton;
          originalUpdateUI = window.textradApp.updateUI;
          originalHandleFormatSelection = window.textradApp.handleFormatSelection;
          
          // Override the hideGenerateButton function to completely prevent hiding
          window.textradApp.hideGenerateButton = function() {
            console.log('Extension prevented generate button from being hidden');
            // Don't call the original function
            
            // Instead, FORCE the button to be visible
            const generateContainer = document.querySelector('#generate-container');
            if (generateContainer) {
              generateContainer.classList.remove('hidden');
              generateContainer.setAttribute('style', 'display: block !important');
            }
            
            const generateButton = document.querySelector('#generate-button');
            if (generateButton) {
              generateButton.classList.remove('hidden');
              generateButton.setAttribute('style', 'display: inline-block !important');
            }
            
            return false;
          };
          
          // Override the showGenerateButton function to ensure it actually shows
          window.textradApp.showGenerateButton = function() {
            console.log('Extension enforcing generate button visibility');
            
            // Call original function
            if (typeof originalShowGenerateButton === 'function') {
              originalShowGenerateButton.apply(this, arguments);
            }
            
            // Also force our own implementation with !important styles
            const generateContainer = document.querySelector('#generate-container');
            if (generateContainer) {
              generateContainer.classList.remove('hidden');
              generateContainer.setAttribute('style', 'display: block !important');
            }
            
            const generateButton = document.querySelector('#generate-button');
            if (generateButton) {
              generateButton.classList.remove('hidden');
              generateButton.setAttribute('style', 'display: inline-block !important');
            }
            
            return true;
          };
          
          // Override the handleFormatSelection function to keep generate button visible
          if (originalHandleFormatSelection) {
            window.textradApp.handleFormatSelection = function() {
              console.log('Extension detected format selection');
              
              // Call the original function first
              if (typeof originalHandleFormatSelection === 'function') {
                originalHandleFormatSelection.apply(this, arguments);
              }
              
              // Then force the generate button to be visible after a delay
              // to let the app's own logic complete
              setTimeout(() => {
                console.log('Forcing generate button visibility after format selection');
                const generateContainer = document.querySelector('#generate-container');
                if (generateContainer) {
                  generateContainer.classList.remove('hidden');
                  generateContainer.setAttribute('style', 'display: block !important');
                }
                
                const generateButton = document.querySelector('#generate-button');
                if (generateButton) {
                  generateButton.classList.remove('hidden');
                  generateButton.setAttribute('style', 'display: inline-block !important');
                }
              }, 100);
            };
          }
          
          // Hook into the updateUI function to ensure our elements stay visible
          if (originalUpdateUI) {
            window.textradApp.updateUI = function() {
              // Call original function
              if (typeof originalUpdateUI === 'function') {
                originalUpdateUI.apply(this, arguments);
              }
              
              // Check if we have transcription content
              const transcriptionText = document.getElementById('transcription-text');
              if (transcriptionText && transcriptionText.value && transcriptionText.value.trim().length > 0) {
                // If we have content and a format is selected, force show the generate button
                const formatSelector = document.getElementById('format-selector');
                if (formatSelector && formatSelector.value && formatSelector.value !== "") {
                  const generateContainer = document.querySelector('#generate-container');
                  if (generateContainer) {
                    console.log('Updating UI - enforcing generate button visibility');
                    generateContainer.classList.remove('hidden');
                    generateContainer.setAttribute('style', 'display: block !important');
                  }
                  
                  const generateButton = document.querySelector('#generate-button');
                  if (generateButton) {
                    generateButton.classList.remove('hidden');
                    generateButton.setAttribute('style', 'display: inline-block !important');
                  }
                }
              }
            };
          }
        }
      }
      
      // Run setup immediately and also check again after a delay
      setupAppOverrides();
      setTimeout(setupAppOverrides, 500);
      setTimeout(setupAppOverrides, 1000);
      
      // Set up a MutationObserver to watch for generate button hiding
      const observer = new MutationObserver((mutations) => {
        mutations.forEach((mutation) => {
          if (mutation.type === 'attributes' && 
              (mutation.attributeName === 'class' || mutation.attributeName === 'style')) {
            const target = mutation.target;
            
            // Check if this is the generate container or button being hidden
            if ((target.id === 'generate-container' || target.id === 'generate-button') && 
                (target.classList.contains('hidden') || target.style.display === 'none')) {
                
              // Check if we have transcription and format
              const transcriptionText = document.getElementById('transcription-text');
              const formatSelector = document.getElementById('format-selector');
              
              if (transcriptionText && 
                  transcriptionText.value && 
                  transcriptionText.value.trim().length > 0 &&
                  formatSelector && 
                  formatSelector.value && 
                  formatSelector.value !== "") {
                
                console.log('Observer detected generate button being hidden, making it visible again');
                setTimeout(() => {
                  target.classList.remove('hidden');
                  target.setAttribute('style', 'display: ' + 
                                     (target.id === 'generate-button' ? 'inline-block' : 'block') + 
                                     ' !important');
                }, 0);
              }
            }
          }
        });
      });
      
      // Start observing
      observer.observe(document.body, { 
        attributes: true, 
        attributeFilter: ['class', 'style'],
        subtree: true,
        childList: true
      });
      
      // Listen for format changes to update the generate button visibility
      const formatSelector = document.getElementById('format-selector');
      if (formatSelector) {
        formatSelector.addEventListener('change', (event) => {
          // Small delay to let the app process the change
          setTimeout(() => {
            const transcriptionText = document.getElementById('transcription-text');
            if (transcriptionText && 
                transcriptionText.value && 
                transcriptionText.value.trim().length > 0 &&
                event.target.value && 
                event.target.value !== "") {
              
              console.log('Format changed, forcing generate button visibility');
              
              // Show generate button with !important styles
              const generateContainer = document.querySelector('#generate-container');
              if (generateContainer) {
                generateContainer.classList.remove('hidden');
                generateContainer.setAttribute('style', 'display: block !important');
              }
              
              const generateButton = document.querySelector('#generate-button');
              if (generateButton) {
                generateButton.classList.remove('hidden');
                generateButton.setAttribute('style', 'display: inline-block !important');
              }
            }
          }, 100);
          
          // Check again after a longer delay in case app's logic runs later
          setTimeout(() => {
            const transcriptionText = document.getElementById('transcription-text');
            if (transcriptionText && 
                transcriptionText.value && 
                transcriptionText.value.trim().length > 0 &&
                event.target.value && 
                event.target.value !== "") {
              
              console.log('Format changed (delayed check), forcing generate button visibility');
              
              // Show generate button with !important styles
              const generateContainer = document.querySelector('#generate-container');
              if (generateContainer) {
                generateContainer.classList.remove('hidden');
                generateContainer.setAttribute('style', 'display: block !important');
              }
              
              const generateButton = document.querySelector('#generate-button');
              if (generateButton) {
                generateButton.classList.remove('hidden');
                generateButton.setAttribute('style', 'display: inline-block !important');
              }
            }
          }, 500);
        });
      }
      
      // Announce bridge presence to web app
      document.dispatchEvent(new CustomEvent('textrad-extension-bridge-ready'));
      
      // Announce bridge presence to extension
      document.dispatchEvent(new CustomEvent('textrad-webapp-message', {
        detail: {
          type: 'APP_READY',
          payload: {
            url: window.location.href,
            title: document.title,
            timestamp: Date.now()
          }
        }
      }));
    })();
  `;
  
  // Append to document
  (document.head || document.documentElement).appendChild(script);
  
  // Clean up
  script.remove();
}

/**
 * Inject transcription into TextRad UI
 */
function injectTranscription(transcription) {
  console.log('Attempting to inject transcription:', transcription.substring(0, 50) + '...');
  
  // Refresh UI elements if needed
  if (!transcriptionElement) {
    findUIElements();
  }
  
  // Check if we still can't find the transcription element
  if (!transcriptionElement) {
    console.error('Transcription element not found, trying one more scan with delay');
    
    // Try again after a short delay (UI might still be initializing)
    setTimeout(() => {
      findUIElements();
      
      if (!transcriptionElement) {
        console.error('Transcription element still not found after retry');
        return false;
      } else {
        // Found it on retry, now inject
        performInjection(transcription);
      }
    }, 500);
    
    return false;
  }
  
  return performInjection(transcription);
}

/**
 * Actually perform the transcription injection
 */
function performInjection(transcription) {
  // Try to use the "type a report" flow instead of dictation flow
  // This should prevent the generate button from disappearing
  if (tryTypeReportFlow(transcription)) {
    console.log('Successfully used Type Report flow for injection');
    return true;
  }
  
  // If Type Report flow fails, fall back to the original method
  console.log('Falling back to standard transcription injection flow');
  
  // First, make sure the transcription container is visible in the web app
  showTranscriptionContainer();
  
  // Inject transcription
  try {
    if (transcriptionElement.tagName === 'TEXTAREA' || transcriptionElement.tagName === 'INPUT') {
      // For textarea/input elements
      transcriptionElement.value = transcription;
      
      // Trigger input event
      transcriptionElement.dispatchEvent(new Event('input', { bubbles: true }));
      
      // Trigger change event
      transcriptionElement.dispatchEvent(new Event('change', { bubbles: true }));
      
      // Focus the element to ensure TextRad app registers the change
      transcriptionElement.focus();
    } else if (transcriptionElement.isContentEditable) {
      // For contenteditable elements
      transcriptionElement.innerHTML = transcription.replace(/\n/g, '<br>');
      
      // Trigger input event
      transcriptionElement.dispatchEvent(new InputEvent('input', { bubbles: true }));
      
      // Focus the element
      transcriptionElement.focus();
    } else if (transcriptionElement.classList.contains('ProseMirror') || 
               transcriptionElement.classList.contains('ql-editor') ||
               transcriptionElement.classList.contains('CodeMirror') ||
               transcriptionElement.classList.contains('monaco-editor')) {
      // Handle special editor frameworks
      
      // Set content - first try innerHTML
      transcriptionElement.innerHTML = transcription.replace(/\n/g, '<br>');
      
      // Trigger both input and change events to ensure the framework detects the change
      transcriptionElement.dispatchEvent(new InputEvent('input', { bubbles: true }));
      transcriptionElement.dispatchEvent(new Event('change', { bubbles: true }));
      
      // If ProseMirror, try its API
      if (transcriptionElement.classList.contains('ProseMirror') && window.ProseMirror) {
        try {
          const view = window.ProseMirror.findView(transcriptionElement);
          if (view && view.dispatch) {
            // This is a simplified approach - actual implementation would depend on ProseMirror setup
            view.dispatch(view.state.tr.insertText(transcription));
          }
        } catch (e) {
          console.log('ProseMirror API access failed, falling back to DOM manipulation:', e);
        }
      }
      
      // Focus the element
      transcriptionElement.focus();
    } else {
      // Fallback to direct property setting
      transcriptionElement.textContent = transcription;
      
      // Also try setting innerHTML as a fallback
      try {
        transcriptionElement.innerHTML = transcription.replace(/\n/g, '<br>');
      } catch (e) {
        console.log('innerHTML setting failed, already tried textContent');
      }
      
      // Try dispatching events anyway
      try {
        transcriptionElement.dispatchEvent(new InputEvent('input', { bubbles: true }));
        transcriptionElement.dispatchEvent(new Event('change', { bubbles: true }));
      } catch (e) {
        console.log('Event dispatching failed for this element type');
      }
    }
    
    // Trigger custom event for TextRad app
    document.dispatchEvent(new CustomEvent('textrad-transcription-received', {
      detail: { transcription }
    }));
    
    // Also dispatch an additional custom event for the web app
    // This helps ensure the app's own code responds to the transcription
    document.dispatchEvent(new CustomEvent('transcription-ready', {
      detail: { text: transcription, source: 'extension' }
    }));
    
    // Make sure the generate button appears
    const generateContainer = document.querySelector('#generate-container');
    if (generateContainer && generateContainer.classList.contains('hidden')) {
      console.log('Making Generate Report button visible');
      generateContainer.classList.remove('hidden');
    }
    
    // Try to use the app's own function to show the generate button if available
    if (window.textradApp && typeof window.textradApp.showGenerateButton === 'function') {
      console.log('Calling app\'s showGenerateButton function');
      window.textradApp.showGenerateButton();
    }
    
    // Also try to dispatch a custom event that might trigger the app to show the button
    document.dispatchEvent(new CustomEvent('transcription-content-updated', {
      detail: { hasContent: true }
    }));
    
    // Set up repeated checks to ensure the generate button stays visible
    // First immediate check
    ensureGenerateButtonVisible();
    
    // Then check again a few times with increasing delays
    setTimeout(ensureGenerateButtonVisible, 500);
    setTimeout(ensureGenerateButtonVisible, 1000);
    setTimeout(ensureGenerateButtonVisible, 2000);
    
    // Set up a continuous monitoring interval for the generate button
    // This ensures it's always visible even if the app tries to hide it
    startGenerateButtonMonitoring();
    
    // Also set up a listener for format selector changes to re-show the button
    const formatSelector = document.querySelector('#format-selector');
    if (formatSelector && !formatSelector.hasChangeListener) {
      formatSelector.addEventListener('change', () => {
        console.log('Format changed, ensuring generate button visibility');
        // Give the app a moment to process the change
        setTimeout(ensureGenerateButtonVisible, 100);
        setTimeout(ensureGenerateButtonVisible, 500);
      });
      formatSelector.hasChangeListener = true;
    }
    
    console.log('Transcription injected successfully');
    return true;
  } catch (error) {
    console.error('Error injecting transcription:', error);
    return false;
  }
}

/**
 * Try to use the "Type a report" flow instead of dictation
 * According to the user, this flow doesn't have the issue with the generate button disappearing
 */
function tryTypeReportFlow(transcription) {
  try {
    console.log('Attempting to use Type Report flow...');
    
    // 1. Find the "Type a report" or "Type directly" button/link
    const typeButtonSelectors = [
      '#type-directly-button',
      'button:contains("Type directly")',
      'button:contains("Type a report")',
      'a:contains("Type directly")',
      'a:contains("Type a report")',
      '.type-report-btn',
      '.type-directly-btn'
    ];
    
    let typeButton = null;
    
    // Try each selector
    for (const selector of typeButtonSelectors) {
      if (selector.includes(':contains')) {
        // Handle custom :contains selector
        const containsText = selector.match(/:contains\("(.+?)"\)/)[1];
        const elements = document.querySelectorAll(selector.split(':')[0]);
        for (const el of elements) {
          if (el.textContent.includes(containsText)) {
            typeButton = el;
            break;
          }
        }
      } else {
        // Regular selector
        typeButton = document.querySelector(selector);
      }
      
      if (typeButton) {
        console.log(`Found Type Report button using selector: ${selector}`);
        break;
      }
    }
    
    // If we couldn't find the button, try looking through all buttons
    if (!typeButton) {
      const allButtons = document.querySelectorAll('button, a');
      for (const button of allButtons) {
        if (button.textContent.toLowerCase().includes('type') || 
            button.innerText.toLowerCase().includes('type') ||
            button.textContent.toLowerCase().includes('manually') ||
            button.innerText.toLowerCase().includes('manually')) {
          typeButton = button;
          console.log('Found Type Report button by scanning all buttons:', button);
          break;
        }
      }
    }
    
    if (!typeButton) {
      console.log('Could not find Type Report button, aborting Type Report flow');
      return false;
    }
    
    // 2. Click the button to switch to the "Type a report" flow
    console.log('Clicking Type Report button...');
    typeButton.click();
    
    // 3. Wait a moment for the UI to update
    // Use a promise to make this function awaitable
    return new Promise((resolve) => {
      setTimeout(() => {
        // 4. Find the transcription textarea in this flow
        const typeTextareaSelectors = [
          '#transcription-text',
          '#type-text',
          '#report-text',
          'textarea',
          '[contenteditable="true"]'
        ];
        
        let typeTextarea = null;
        
        for (const selector of typeTextareaSelectors) {
          typeTextarea = document.querySelector(selector);
          if (typeTextarea) {
            console.log(`Found Type Report textarea using selector: ${selector}`);
            break;
          }
        }
        
        if (!typeTextarea) {
          console.log('Could not find Type Report textarea, aborting Type Report flow');
          resolve(false);
          return;
        }
        
        // 5. Inject the transcription into the textarea
        try {
          console.log('Injecting transcription into Type Report textarea...');
          
          if (typeTextarea.tagName === 'TEXTAREA' || typeTextarea.tagName === 'INPUT') {
            typeTextarea.value = transcription;
            typeTextarea.dispatchEvent(new Event('input', { bubbles: true }));
            typeTextarea.dispatchEvent(new Event('change', { bubbles: true }));
          } else if (typeTextarea.isContentEditable) {
            typeTextarea.innerHTML = transcription.replace(/\n/g, '<br>');
            typeTextarea.dispatchEvent(new InputEvent('input', { bubbles: true }));
          } else {
            typeTextarea.textContent = transcription;
            try {
              typeTextarea.innerHTML = transcription.replace(/\n/g, '<br>');
            } catch (e) { /* Ignore */ }
            
            try {
              typeTextarea.dispatchEvent(new InputEvent('input', { bubbles: true }));
              typeTextarea.dispatchEvent(new Event('change', { bubbles: true }));
            } catch (e) { /* Ignore */ }
          }
          
          // Focus the textarea
          typeTextarea.focus();
          
          // 6. Dispatch events to ensure the app processes the change
          document.dispatchEvent(new CustomEvent('textrad-transcription-received', {
            detail: { transcription }
          }));
          
          document.dispatchEvent(new CustomEvent('transcription-ready', {
            detail: { text: transcription, source: 'extension' }
          }));
          
          // Log success
          console.log('Successfully injected transcription using Type Report flow');
          resolve(true);
        } catch (error) {
          console.error('Error injecting into Type Report textarea:', error);
          resolve(false);
        }
      }, 500); // Wait for UI to update
    });
  } catch (error) {
    console.error('Error during Type Report flow:', error);
    return false;
  }
}

/**
 * Show the transcription container and necessary UI elements
 */
function showTranscriptionContainer() {
  console.log('Ensuring transcription container is visible');
  
  // Step 1: Make sure we're in the correct section of the app
  // First, check if we need to open the recorder section
  const recorderSection = document.getElementById('recorder-section');
  const reportsSection = document.getElementById('reports-section');
  
  // If the recorder section is hidden, we need to show it
  if (recorderSection && recorderSection.classList.contains('hidden')) {
    console.log('Opening recorder section');
    
    // Try to find and click the "New Report" button to show the recorder
    const newReportButton = Array.from(document.querySelectorAll('button')).find(
      button => button.textContent.includes('New Report') || 
                button.innerHTML.includes('fa-plus')
    );
    
    if (newReportButton) {
      console.log('Found New Report button, clicking it');
      newReportButton.click();
    } else {
      // Manual show/hide if button not found
      console.log('New Report button not found, manually showing recorder section');
      if (reportsSection) reportsSection.classList.add('hidden');
      recorderSection.classList.remove('hidden');
    }
    
    // Give the UI a moment to update
    setTimeout(() => ensureTranscriptionContainerVisible(), 100);
  } else {
    // We're already in the recorder section, just ensure the transcription container is visible
    ensureTranscriptionContainerVisible();
  }
}

/**
 * Ensure the transcription container and related UI elements are visible
 */
function ensureTranscriptionContainerVisible() {
  // Step 2: Make sure the transcription container and related UI elements are visible
  const containers = [
    { selector: '#transcription-container', name: 'Transcription container' },
    { selector: '.transcription-container', name: 'Transcription container (class)' },
    { selector: '#format-selector-container', name: 'Format selector' },
    { selector: '#generate-container', name: 'Generate button container' },
    { selector: '#generate-button', name: 'Generate button' },
    { selector: '.recorder-container', name: 'Recorder container' }
  ];
  
  for (const { selector, name } of containers) {
    const container = document.querySelector(selector);
    if (container) {
      if (container.classList.contains('hidden')) {
        console.log(`Un-hiding ${name}`);
        container.classList.remove('hidden');
      } else {
        console.log(`${name} is already visible`);
      }
    }
  }
  
  // Step 3: Make sure the format selector is visible, but don't auto-select any format
  const formatSelectorContainer = document.querySelector('#format-selector-container');
  if (formatSelectorContainer && formatSelectorContainer.classList.contains('hidden')) {
    console.log('Making format selector visible');
    formatSelectorContainer.classList.remove('hidden');
  }
  
  // Do NOT auto-select any format - leave it to the user to choose
}

/**
 * Ensure the generate button is visible
 * This function forces the generate button to appear regardless of app state
 */
function ensureGenerateButtonVisible() {
  const generateContainer = document.querySelector('#generate-container');
  if (generateContainer) {
    if (generateContainer.classList.contains('hidden')) {
      console.log('Making Generate Report button container visible');
      generateContainer.classList.remove('hidden');
    }
    
    // Also ensure the button itself is visible
    const generateButton = document.querySelector('#generate-button');
    if (generateButton && generateButton.classList.contains('hidden')) {
      console.log('Making Generate Report button visible');
      generateButton.classList.remove('hidden');
    }
    
    // If there's an inline style hiding it, remove that too
    if (generateContainer.style.display === 'none') {
      generateContainer.style.display = 'block';
    }
    
    // Force the container to be shown with !important flag via style attribute
    generateContainer.setAttribute('style', 'display: block !important');
    
    // Also force the button to be shown with !important
    if (generateButton) {
      generateButton.setAttribute('style', 'display: inline-block !important');
    }
  }
  
  // Try triggering the app's internal functions if available
  if (window.textradApp) {
    if (typeof window.textradApp.showGenerateButton === 'function') {
      window.textradApp.showGenerateButton();
    }
    
    if (typeof window.textradApp.enableGenerateButton === 'function') {
      window.textradApp.enableGenerateButton();
    }
    
    if (typeof window.textradApp.updateUI === 'function') {
      window.textradApp.updateUI();
    }
    
    // Add button back to DOM if it was removed
    if (!document.querySelector('#generate-button')) {
      console.log('Generate button not found in DOM, recreating it');
      const newButton = document.createElement('button');
      newButton.id = 'generate-button';
      newButton.className = 'bg-blue-600 hover:bg-blue-700 text-white font-medium py-2 px-4 rounded-md';
      newButton.innerHTML = 'Generate Report';
      
      // Add the event listener that would generate a report
      newButton.addEventListener('click', () => {
        if (window.textradApp && typeof window.textradApp.generateReport === 'function') {
          window.textradApp.generateReport();
        }
      });
      
      // Create container if it doesn't exist
      if (!generateContainer) {
        const newContainer = document.createElement('div');
        newContainer.id = 'generate-container';
        newContainer.className = 'mt-4';
        newContainer.setAttribute('style', 'display: block !important');
        newContainer.appendChild(newButton);
        
        // Add it to the transcription container
        const transcriptionContainer = document.querySelector('.transcription-container');
        if (transcriptionContainer) {
          transcriptionContainer.appendChild(newContainer);
        }
      } else {
        generateContainer.appendChild(newButton);
      }
    }
  }
  
  // Dispatch events that might trigger the app
  document.dispatchEvent(new CustomEvent('format-selected', { detail: { hasValidFormat: true } }));
  document.dispatchEvent(new CustomEvent('transcription-ready', { detail: { hasContent: true } }));
}

// Initialize content script
initialize();