/**
 * TextRad Dictation Extension - Direct Transcription Service
 * 
 * This service bypasses the Firebase client libraries and directly
 * calls the TextRad server-side transcription endpoint.
 */

export class DirectTranscriptionService {
  constructor() {
    // TextRad transcription endpoint - HTTP endpoint for CORS-enabled function
    this.transcriptionEndpoint = 'https://asia-south1-textrad-dc865.cloudfunctions.net/directTranscribeHttp';
    
    // Event callbacks
    this.onTranscriptionStart = null;
    this.onTranscriptionComplete = null;
    this.onTranscriptionError = null;
    this.onTranscriptionProgress = null;
    
    // Store current transcription for appending when using 'Record More'
    this.currentTranscription = '';
  }
  
  /**
   * Transcribe audio using direct API call
   * @param {Blob} audioBlob - The recorded audio blob
   */
  async transcribeAudio(audioBlob) {
    console.log('Starting direct transcription service');
    
    // Call start callback
    if (typeof this.onTranscriptionStart === 'function') {
      this.onTranscriptionStart();
    }
    
    try {
      // Convert audio to base64
      const base64Audio = await this.blobToBase64(audioBlob);
      console.log("Audio converted to base64, size:", Math.round(base64Audio.length / 1024), "KB");
      
      if (typeof this.onTranscriptionProgress === 'function') {
        this.onTranscriptionProgress(0.3);
      }
      
      // Call the transcription API directly
      const response = await this.callTranscriptionAPI(base64Audio);
      
      if (typeof this.onTranscriptionProgress === 'function') {
        this.onTranscriptionProgress(0.9);
      }
      
      // Process the response
      if (!response || !response.text) {
        throw new Error('Invalid response from transcription service');
      }
      
      const transcription = response.text;
      
      // We'll let the popup.js handle appending to avoid duplication
      // Just store the current transcription property for reference
      
      // Call completion callback with ONLY the new transcription
      // The parent component will handle appending if needed
      if (typeof this.onTranscriptionComplete === 'function') {
        this.onTranscriptionComplete(transcription);
      }
      
      return transcription;
    } catch (error) {
      console.error('Transcription error:', error);
      
      // Call error callback
      if (typeof this.onTranscriptionError === 'function') {
        this.onTranscriptionError(error);
      }
      
      throw error;
    }
  }
  
  /**
   * Call the TextRad transcription API directly
   * @param {string} base64Audio - Base64-encoded audio data
   * @returns {Promise<Object>} - Transcription response
   */
  async callTranscriptionAPI(base64Audio) {
    // Prepare the request - match the exact format expected by your function
    const requestData = {
      data: {
        audioData: base64Audio,
        userId: 'extension-user', // Anonymous user ID
        source: 'extension' // Identify source as extension
      }
    };
    
    console.log('Calling API with data structure:', Object.keys(requestData));
    
    try {
      // Make the request - Firebase Functions expect a specific format
      const response = await fetch(this.transcriptionEndpoint, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-TextRad-Extension': 'true',
          'Origin': 'chrome-extension://textrad-extension'
        },
        body: JSON.stringify(requestData)
      });
      
      // Check for HTTP errors
      if (!response.ok) {
        throw new Error(`HTTP error ${response.status}: ${response.statusText}`);
      }
      
      // Parse the response
      const data = await response.json();
      return data.result || data; // Handle response format from HTTP function
    } catch (error) {
      console.error('API call failed:', error);
      
      // No more fallbacks - always throw the error
      if (error.message.includes('Failed to fetch')) {
        console.warn('Network error accessing transcription API');
        throw new Error('Network error occurred. Please check your connection and ensure you are logged in to TextRad.');
      }
      
      throw error;
    }
  }
  
  /**
   * Convert Blob to Base64 string
   * @param {Blob} blob - The blob to convert
   * @returns {Promise<string>} Base64 string
   */
  blobToBase64(blob) {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.onloadend = () => {
        // Get the base64 string (remove the data URL prefix)
        const base64String = reader.result.split(',')[1];
        resolve(base64String);
      };
      reader.onerror = reject;
      reader.readAsDataURL(blob);
    });
  }
  
  /**
   * Set the current transcription (used for appending with 'Record More')
   * @param {string} text - The transcription text to store
   */
  setCurrentTranscription(text) {
    this.currentTranscription = text || '';
    console.log('Stored current transcription for appending, length:', this.currentTranscription.length);
  }
  
  /**
   * Reset the current transcription
   */
  resetTranscription() {
    this.currentTranscription = '';
    console.log('Reset current transcription');
  }
}