/**
 * TextRad Dictation Extension - Popup Script
 * 
 * Responsibilities:
 * - User interface for recording and managing dictations
 * - Communication with service worker (background script)
 * - Audio recording and visualization
 * - Displaying transcription results
 */

// Import AudioRecorder and TranscriptionService
import { ExtensionAudioRecorder } from '../lib/audio-recorder.js';
import { ExtensionTranscriptionService } from '../lib/transcription-service.js';
import { StorageManager } from '../lib/storage-manager.js';

// UI Elements
let recordButton;
let visualizerCanvas;
let recordingStatus;
// Timer removed
let transcriptionPanel;
let transcriptionText;
let newDictationButton;
let copyButton;
let sendButton;
let settingsButton;
let settingsPanel;
let closeSettingsButton;
let saveSettingsButton;
let appStatusPanel;
let appStatusIndicator;
let appStatusText;
let openTextRadButton;
let loadingOverlay;
let loadingText;
let loginRequiredMessage;
let loginButton;
let toastNotification;
let toastMessage;

// Main/Mini UI Elements
let expandedContainer;
let collapsedContainer;
let collapseButton;
let expandButton;
let miniRecordButton;
let closeButton;
let collapsedStatus;

// Settings Elements
let autoSendSetting;
let silenceTimeoutSetting;
let silenceTimeoutValue;

// Service instances
let audioRecorder;
let transcriptionService;
let storageManager;

// State tracking
let isRecording = false;
let isTranscribing = false;
let recordingStartTime = 0;
let recordingTimer = null;
let currentTranscription = '';
let textradTabs = [];
let selectedTabId = null;

// Flag to determine whether to append or replace transcription
// true = append to existing transcription (set by main record button)
// false = start fresh (set by New Dictation button)
// Default to undefined so we know when it's not been explicitly set
let recordMoreButtonClicked = undefined;

/**
 * Initialize popup
 */
async function initialize() {
  console.log('TextRad Dictation Extension popup initialized');
  
  // Initialize services
  storageManager = new StorageManager();
  await storageManager.initialize();
  
  // Get UI elements
  getUIElements();
  
  // Set up event listeners
  setupEventListeners();
  
  // Initialize audio recorder and transcription service
  initializeServices();
  
  // No need to move buttons anymore - using simpler approach
  
  // Check for TextRad tabs
  findTextRadTabs();
  
  // Load settings
  loadSettings();
  
  // Reset record mode flags to ensure clean start
  recordMoreButtonClicked = undefined;
  
  // Check URL for mini mode parameter
  const urlParams = new URLSearchParams(window.location.search);
  if (urlParams.has('viewMode') && urlParams.get('viewMode') === 'mini') {
    // Switch to mini mode immediately from URL param
    expandedContainer.classList.add('hidden');
    collapsedContainer.classList.remove('hidden');
    document.body.classList.add('mini-mode');
    document.documentElement.classList.add('mini-mode');
    
    // Force window size to exactly match mini UI
    window.resizeTo(150, 40);
    
    updateMiniStatus();
  } else {
    // Otherwise, load saved view mode
    chrome.storage.local.get(['viewMode'], (result) => {
      if (result.viewMode === 'mini') {
        // Switch to mini mode
        expandedContainer.classList.add('hidden');
        collapsedContainer.classList.remove('hidden');
        document.body.classList.add('mini-mode');
        document.documentElement.classList.add('mini-mode');
        
        // Force window size to exactly match mini UI
        window.resizeTo(140, 32);
        
        updateMiniStatus();
      } else {
        // Default to expanded mode
        expandedContainer.classList.remove('hidden');
        collapsedContainer.classList.add('hidden');
        document.body.classList.remove('mini-mode');
        document.documentElement.classList.remove('mini-mode');
      }
    });
  }
  
  // Check state of recording from service worker
  const extensionState = await chrome.runtime.sendMessage({ action: 'GET_STATE' });
  console.log('Extension state:', extensionState);
  
  // Check for any stale recording state and reset it
  if (extensionState && extensionState.state && extensionState.state.recording) {
    console.log('Found stale recording state, resetting it');
    
    // Reset recording state in the service worker
    chrome.runtime.sendMessage({ 
      action: 'SET_TRANSCRIPTION', 
      transcription: '' // Empty string to clear transcription
    });
    
    // Also reset recording state in storage
    await storageManager.setRecordingInProgress(false);
    
    // Show default recording UI
    isRecording = false;
    showRecordingUI();
  } else if (extensionState && extensionState.state && extensionState.state.transcribing) {
    console.log('Found stale transcribing state, resetting it');
    
    // Reset the state in the service worker
    chrome.runtime.sendMessage({ 
      action: 'SET_TRANSCRIPTION', 
      transcription: '' // Empty string to clear transcription
    });
    
    // Show default recording UI
    isRecording = false;
    showRecordingUI();
  } else if (extensionState && extensionState.state && extensionState.state.lastTranscription) {
    // If we have a previous transcription, show it
    console.log('Found previous transcription, showing it');
    currentTranscription = extensionState.state.lastTranscription;
    showTranscriptionUI(currentTranscription);
  }
  
  // Check extension state
  checkState();
}

/**
 * Get UI elements
 */
function getUIElements() {
  // Main UI elements
  recordButton = document.getElementById('record-button');
  visualizerCanvas = null; // Removing visualizer
  recordingStatus = document.getElementById('recording-status');
  // Timer element removed
  transcriptionPanel = document.getElementById('transcription-panel');
  transcriptionText = document.getElementById('transcription-text');
  newDictationButton = document.getElementById('new-dictation-button');
  copyButton = document.getElementById('copy-button');
  sendButton = document.getElementById('send-button');
  
  // App status elements
  appStatusPanel = document.getElementById('app-status-panel');
  appStatusIndicator = document.getElementById('app-status-indicator').querySelector('.status-dot');
  appStatusText = document.getElementById('app-status-text');
  openTextRadButton = document.getElementById('open-textrad-button');
  
  // Login required message elements
  loginRequiredMessage = document.getElementById('login-required-message');
  loginButton = document.getElementById('login-button');
  
  // Settings elements
  settingsButton = document.getElementById('settings-button');
  settingsPanel = document.getElementById('settings-panel');
  closeSettingsButton = document.getElementById('close-settings-button');
  saveSettingsButton = document.getElementById('save-settings-button');
  autoSendSetting = document.getElementById('auto-send-setting');
  silenceTimeoutSetting = document.getElementById('silence-timeout-setting');
  silenceTimeoutValue = document.getElementById('silence-timeout-value');
  
  // Loading elements
  loadingOverlay = document.getElementById('loading-overlay');
  loadingText = document.getElementById('loading-text');
  
  // Toast notification elements
  toastNotification = document.getElementById('toast-notification');
  toastMessage = document.getElementById('toast-message');
  
  // Main/Mini UI elements
  expandedContainer = document.getElementById('expanded-container');
  collapsedContainer = document.getElementById('collapsed-container');
  collapseButton = document.getElementById('collapse-button');
  expandButton = document.getElementById('expand-button');
  miniRecordButton = document.getElementById('mini-record-button');
  closeButton = document.getElementById('close-button');
  collapsedStatus = document.getElementById('collapsed-status');
}

/**
 * Set up event listeners
 */
function setupEventListeners() {
  // Record button
  recordButton.addEventListener('click', toggleRecording);
  
  // New dictation button
  newDictationButton.addEventListener('click', startNewDictation);
  
  // Copy button
  copyButton.addEventListener('click', copyTranscription);
  
  // Send button
  sendButton.addEventListener('click', sendTranscription);
  
  // Open TextRad button
  openTextRadButton.addEventListener('click', openTextRad);
  
  // Login button (if present)
  if (loginButton) {
    loginButton.addEventListener('click', openTextRad);
  }
  
  // Refresh status button
  const refreshButton = document.getElementById('refresh-status-button');
  if (refreshButton) {
    refreshButton.addEventListener('click', () => {
      // Show loading indicator
      showLoading('Refreshing login status...');
      
      // Call findTextRadTabs again to check login status
      findTextRadTabs();
    });
  }
  
  // Settings button
  settingsButton.addEventListener('click', toggleSettings);
  
  // Close settings button
  closeSettingsButton.addEventListener('click', toggleSettings);
  
  // Save settings button
  saveSettingsButton.addEventListener('click', saveSettings);
  
  // Silence timeout slider
  silenceTimeoutSetting.addEventListener('input', updateSilenceTimeoutValue);
  
  // Collapse button to switch to mini UI
  collapseButton.addEventListener('click', toggleViewMode);
  
  // Mini record button in mini mode
  miniRecordButton.addEventListener('click', toggleRecording);
  
  // Expand button to go back to full view
  expandButton.addEventListener('click', toggleViewMode);
  
  // Close button - actually closes the popup
  closeButton.addEventListener('click', closePopup);
  
  // Add beforeunload event listener to handle potential popup closing
  window.addEventListener('beforeunload', handlePopupClosing);
  
  // Handle fullscreen/mini toggle via keyboard shortcut (Escape key)
  document.addEventListener('keydown', (e) => {
    if (e.key === 'Escape') {
      toggleViewMode();
      e.preventDefault();
    }
  });
}

/**
 * Initialize audio recorder and transcription service
 */
function initializeServices() {
  // Initialize audio recorder without visualizer
  audioRecorder = new ExtensionAudioRecorder(null, storageManager);
  
  // Set audio recorder event handlers
  audioRecorder.onRecordingStart = handleRecordingStart;
  audioRecorder.onRecordingStop = handleRecordingStop;
  audioRecorder.onSilenceDetected = handleSilenceDetected;
  audioRecorder.onError = handleRecordingError;
  
  // Initialize transcription service
  transcriptionService = new ExtensionTranscriptionService(storageManager);
  
  // Set transcription service event handlers
  transcriptionService.onTranscriptionStart = handleTranscriptionStart;
  transcriptionService.onTranscriptionComplete = handleTranscriptionComplete;
  transcriptionService.onTranscriptionError = handleTranscriptionError;
  
  // Initialize Firebase services
  transcriptionService.initializeFirebaseServices();
}

/**
 * Check for TextRad tabs
 */
function findTextRadTabs() {
  updateAppStatus('connecting');
  
  // Show loading while checking
  showLoading('Checking TextRad connection...');
  
  // Set a timeout to hide the loading indicator if it takes too long
  const loadingTimeout = setTimeout(() => {
    console.log('Loading timeout reached, hiding loading indicator');
    hideLoading();
  }, 5000); // 5 seconds timeout
  
  chrome.runtime.sendMessage({ action: 'FIND_TEXTRAD_TABS' }, (response) => {
    // Always hide loading indicator
    hideLoading();
    
    // Clear the timeout
    clearTimeout(loadingTimeout);
    
    if (chrome.runtime.lastError) {
      console.error('Error finding TextRad tabs:', chrome.runtime.lastError);
      updateAppStatus('disconnected');
      return;
    }
    
    if (response && response.success) {
      // Check if there's a login error message
      if (response.error && response.error.includes('not logged in')) {
        console.log('User not logged in to TextRad');
        updateAppStatus('not-logged-in');
        return;
      }
      
      if (response.tabs && response.tabs.length > 0) {
        textradTabs = response.tabs;
        selectedTabId = textradTabs[0].id;
        
        // Check if these are actually logged-in tabs
        const hasLoggedInTabs = textradTabs.some(tab => 
          response.loggedInTabs && response.loggedInTabs[tab.id] === true
        );
        
        if (hasLoggedInTabs) {
          console.log('Found logged-in TextRad tabs');
          updateAppStatus('connected');
        } else {
          // We found TextRad tabs, but user isn't logged in
          console.log('TextRad tabs found but not logged in');
          updateAppStatus('not-logged-in');
        }
      } else {
        // No TextRad tabs found
        if (response.hasTextRadTabs && response.notLoggedInCount > 0) {
          // We found TextRad tabs, but user isn't logged in
          console.log('TextRad tabs found but not logged in');
          updateAppStatus('not-logged-in');
        } else {
          // No TextRad tabs at all
          console.log('No TextRad tabs found');
          updateAppStatus('disconnected');
        }
      }
    } else {
      console.log('Find TextRad tabs failed or returned no success');
      updateAppStatus('disconnected');
    }
  });
}

/**
 * Update app connection status
 */
function updateAppStatus(status) {
  appStatusIndicator.className = 'status-dot';
  
  // Ensure elements exist before accessing them
  // Hide login required message by default
  const msgElement = document.getElementById('login-required-message');
  if (msgElement) {
    msgElement.classList.add('hidden');
    loginRequiredMessage = msgElement; // Ensure the variable is assigned
  }
  
  // Ensure login button is defined
  if (!loginButton) {
    loginButton = document.getElementById('login-button');
  }
  
  // Disable record button by default
  if (recordButton) {
    recordButton.disabled = false;
  }
  
  switch (status) {
    case 'connected':
      appStatusIndicator.classList.add('connected');
      appStatusText.textContent = 'Connected to TextRad';
      break;
      
    case 'connecting':
      appStatusIndicator.classList.add('connecting');
      appStatusText.textContent = 'Connecting...';
      break;
      
    case 'not-logged-in':
      // Special case - user needs to log in
      appStatusIndicator.classList.add('disconnected');
      appStatusText.textContent = 'Login required';
      
      // Show login message
      const loginMsg = document.getElementById('login-required-message');
      if (loginMsg) {
        loginMsg.classList.remove('hidden');
        loginRequiredMessage = loginMsg; // Ensure the variable is assigned
      }
      
      // Make sure recording UI is fully visible but disabled
      if (recordingPanel) {
        recordingPanel.classList.remove('hidden');
      }
      
      if (transcriptionPanel) {
        transcriptionPanel.classList.add('hidden');
      }
      
      // Ensure login button is set up
      const loginBtn = document.getElementById('login-button');
      if (loginBtn && !loginBtn.hasEventListener) {
        loginBtn.addEventListener('click', openTextRad);
        loginBtn.hasEventListener = true;
        loginButton = loginBtn;
      }
      
      // Ensure refresh button is set up
      const refreshBtn = document.getElementById('refresh-status-button');
      if (refreshBtn && !refreshBtn.hasEventListener) {
        refreshBtn.addEventListener('click', () => {
          showLoading('Refreshing login status...');
          findTextRadTabs();
        });
        refreshBtn.hasEventListener = true;
      }
      
      // Disable recording when not logged in
      if (recordButton) {
        recordButton.disabled = true;
        recordButton.classList.add('disabled');
      }
      break;
      
    case 'disconnected':
    default:
      appStatusIndicator.classList.add('disconnected');
      appStatusText.textContent = 'Not connected';
      break;
  }
  
  // Update mini mode status
  updateMiniStatus();
}

/**
 * Open TextRad web app
 */
function openTextRad() {
  // Open TextRad in a new tab (use root URL to ensure login page is shown if needed)
  chrome.tabs.create({ url: 'https://textrad.in/' });
  
  // Show a helpful message to the user
  const msgElement = document.getElementById('login-required-message');
  if (msgElement) {
    if (!msgElement.querySelector('.login-instruction')) {
      const instruction = document.createElement('p');
      instruction.className = 'login-instruction mt-1 text-green-300';
      instruction.textContent = 'TextRad opened in a new tab. Please log in there and then try again.';
      msgElement.querySelector('div').appendChild(instruction);
    }
  }
}

/**
 * Copy transcription to clipboard
 */
function copyTranscription() {
  // Check if we have a transcription
  if (!transcriptionText.value) {
    alert('No transcription to copy');
    return;
  }
  
  // Ensure currentTranscription is updated with the textarea value
  // This ensures send will work even after copy
  currentTranscription = transcriptionText.value;
  
  // Copy to clipboard
  navigator.clipboard.writeText(transcriptionText.value).then(() => {
    // Show success message
    const originalText = copyButton.innerHTML;
    copyButton.innerHTML = '<i class="fas fa-check mr-1"></i>Copied!';
    
    // Reset button text after 2 seconds
    setTimeout(() => {
      copyButton.innerHTML = originalText;
    }, 2000);
  }).catch(err => {
    console.error('Could not copy text: ', err);
    alert('Failed to copy: ' + err.message);
  });
}

/**
 * Check extension state
 */
function checkState() {
  chrome.runtime.sendMessage({ action: 'GET_STATE' }, (response) => {
    if (chrome.runtime.lastError) {
      console.error('Error getting state:', chrome.runtime.lastError);
      return;
    }
    
    if (response && response.state) {
      const state = response.state;
      
      // Check recording state
      if (state.recording) {
        // We were recording, but popup was closed
        showRecordingUI();
      } else if (state.transcribing) {
        // We were transcribing
        showTranscribingUI();
      } else if (state.lastTranscription) {
        // We have a transcription
        showTranscriptionUI(state.lastTranscription);
      }
    }
  });
}

/**
 * Toggle recording state
 */
function toggleRecording() {
  // First check if we're connected to TextRad and logged in
  if (!isUserLoggedIn()) {
    console.log('Cannot record: User not logged in to TextRad');
    updateAppStatus('not-logged-in');
    return;
  }
  
  if (isRecording) {
    stopRecording();
  } else {
    // For the main record button, we want to append to existing text
    // Set the flag to indicate we want to append
    recordMoreButtonClicked = true;
    
    // Store current transcription before starting new recording
    if (transcriptionText && transcriptionText.value) {
      const previousTranscription = transcriptionText.value;
      currentTranscription = previousTranscription;
      console.log('Main record button clicked - storing transcription for append:', 
                previousTranscription.substring(0, 50) + '...');
    }
    
    startRecording();
  }
}

/**
 * Check if user is logged in to TextRad
 */
function isUserLoggedIn() {
  // If we have textradTabs, then the user is logged in
  return textradTabs && textradTabs.length > 0;
}

/**
 * Start recording (behavior depends on which button initiated it)
 */
async function startRecording() {
  try {
    // Check login status again right before recording
    if (!isUserLoggedIn()) {
      throw new Error('You must be logged in to TextRad to record and transcribe audio');
    }
    
    // Update UI
    recordButton.disabled = true;
    recordingStatus.textContent = 'Starting...';
    
    // The recordMoreButtonClicked flag is set in toggleRecording (normal record button)
    // and explicitly set to false in startNewDictation
    // So we don't need to do anything else with the flag here
    console.log('Starting recording with append mode:', recordMoreButtonClicked ? 'true' : 'false');
    
    /* 
     * Background recording has been disabled - we are not using it anymore
     * Only use direct recording through the audioRecorder
     */
    
    // If background recording fails, fall back to local recording
    console.log('Falling back to local audio recording');
    const success = await audioRecorder.startRecording();
    
    if (!success) {
      throw new Error('Failed to start recording');
    }
  } catch (error) {
    console.error('Error starting recording:', error);
    
    // If this is a login error, update UI appropriately
    if (error.message.includes('logged in')) {
      updateAppStatus('not-logged-in');
    } else {
      recordingStatus.textContent = 'Error: ' + error.message;
    }
    
    recordButton.disabled = false;
  }
}

/**
 * Send a message with timeout
 */
function sendMessageWithTimeout(message, timeout = 3000) {
  return new Promise((resolve, reject) => {
    const timer = setTimeout(() => {
      reject(new Error(`Message timeout after ${timeout}ms`));
    }, timeout);
    
    chrome.runtime.sendMessage(message, (response) => {
      clearTimeout(timer);
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError);
      } else {
        resolve(response);
      }
    });
  });
}

/**
 * Stop recording
 */
async function stopRecording() {
  try {
    // Update UI
    recordButton.disabled = true;
    recordingStatus.textContent = 'Stopping...';
    
    /* 
     * Background recording has been disabled - we are not using it anymore
     * Only use direct recording through the audioRecorder
     */
    
    // Fall back to local recording if background recording fails
    console.log('Falling back to local audio recorder stop');
    audioRecorder.stopRecording();
    
    // Ensure recording in progress flag is cleared
    if (storageManager) {
      storageManager.setRecordingInProgress(false);
    }
  } catch (error) {
    console.error('Error stopping recording:', error);
    recordingStatus.textContent = 'Error: ' + error.message;
    recordButton.disabled = false;
  }
}

/**
 * Start a brand new dictation (clear previous text)
 */
function startNewDictation() {
  // First check if we're connected to TextRad and logged in
  if (!isUserLoggedIn()) {
    console.log('Cannot start new dictation: User not logged in to TextRad');
    updateAppStatus('not-logged-in');
    return;
  }
  
  // EXPLICITLY Reset flag to indicate we're starting fresh
  recordMoreButtonClicked = false;
  
  // Clear the current transcription (local and in services)
  // IMPORTANT: Set both to empty strings to ensure complete clearing
  currentTranscription = '';
  
  // Reset transcription in all services
  transcriptionService.resetTranscription();
  transcriptionService.setCurrentTranscription('');
  
  // Reset the DirectTranscriptionService if it's being imported
  try {
    // This is an async import but we'll just fire it and let it complete
    import('../lib/direct-transcription-service.js').then(module => {
      const DirectTranscriptionService = module.DirectTranscriptionService;
      const directService = new DirectTranscriptionService();
      directService.resetTranscription();
      directService.setCurrentTranscription('');
    }).catch(err => {
      console.error('Error resetting DirectTranscriptionService:', err);
    });
  } catch (err) {
    console.error('Error importing DirectTranscriptionService:', err);
  }
  
  // Explicitly clear the UI text area
  if (transcriptionText) {
    transcriptionText.value = '';
  }
  
  // Also reset the stored value in the service worker
  chrome.runtime.sendMessage({
    action: 'SET_TRANSCRIPTION',
    transcription: ''
  });
  
  console.log('New Dictation clicked - cleared previous transcription');
  
  // Hide transcription panel
  transcriptionPanel.classList.add('hidden');
  
  // Show recording panel
  document.getElementById('recording-panel').classList.remove('hidden');
  
  // Make sure the full UI is shown before starting a new recording
  const fullRecordingUI = document.getElementById('full-recording-ui');
  if (fullRecordingUI) {
    fullRecordingUI.classList.remove('hidden');
  }
  appStatusPanel.classList.remove('hidden');
  
  // DO NOT start recording automatically - let user press the mic button
  // Just reset the UI to recording-ready state
  recordingStatus.textContent = 'Ready to record';
  recordButton.disabled = false;
  recordButton.classList.remove('recording');
}

/**
 * Send transcription to TextRad
 */
function sendTranscription() {
  // Check if we have a transcription
  if (!currentTranscription) {
    alert('No transcription to send');
    return;
  }
  
  // Check the connection state first
  updateAppStatus('connecting');
  
  // Find TextRad tabs with verified login state
  chrome.runtime.sendMessage({ action: 'FIND_TEXTRAD_TABS' }, (response) => {
    if (chrome.runtime.lastError) {
      console.error('Error finding TextRad tabs:', chrome.runtime.lastError);
      hideLoading();
      updateAppStatus('disconnected');
      alert('Error connecting to TextRad: ' + chrome.runtime.lastError.message);
      return;
    }
    
    // Check for login issues
    if (response && response.error && response.error.includes('not logged in')) {
      console.log('User not logged in to TextRad');
      hideLoading();
      updateAppStatus('not-logged-in');
      return;
    }
    
    // Check if we have logged-in TextRad tabs
    if (response && response.success && response.tabs && response.tabs.length > 0) {
      textradTabs = response.tabs;
      selectedTabId = textradTabs[0].id;
      updateAppStatus('connected');
      
      // Show loading state
      showLoading('Sending to TextRad...');
      
      // Send transcription to TextRad tab
      chrome.runtime.sendMessage({
        action: 'SEND_TRANSCRIPTION_TO_TAB',
        tabId: selectedTabId,
        transcription: currentTranscription
      }, (response) => {
        // Hide loading state
        hideLoading();
        
        if (chrome.runtime.lastError) {
          console.error('Error sending transcription:', chrome.runtime.lastError);
          showToast('Error: ' + chrome.runtime.lastError.message, 3000);
          return;
        }
        
        if (response && response.success) {
          // Success - show subtle toast notification
          showToast('✓ Sent to TextRad', 1500);
          
          // Update button to show temporary success state
          const originalHTML = sendButton.innerHTML;
          sendButton.innerHTML = '<i class="fas fa-check"></i>';
          sendButton.classList.add('bg-green-700');
          
          // Reset button after a delay
          setTimeout(() => {
            sendButton.innerHTML = originalHTML;
            sendButton.classList.remove('bg-green-700');
          }, 1500);
        } else if (response && response.error && response.error.includes('not logged in')) {
          // Login error
          updateAppStatus('not-logged-in');
        } else {
          // Other error
          showToast('Error: ' + (response?.error || 'Unknown error'), 3000);
        }
      });
    } else {
      // No TextRad tabs found or not logged in
      hideLoading();
      
      if (response && response.hasTextRadTabs && response.notLoggedInCount > 0) {
        // Found TextRad tabs but user isn't logged in
        updateAppStatus('not-logged-in');
      } else {
        // No TextRad tabs at all
        updateAppStatus('disconnected');
        
        // Ask if user wants to open TextRad
        if (confirm('No TextRad app detected. Open TextRad in a new tab?')) {
          openTextRad();
        }
      }
    }
  });
}

/**
 * Handle recording start
 */
function handleRecordingStart() {
  console.log('Recording started');
  
  // Update state
  isRecording = true;
  recordingStartTime = Date.now();
  
  // Update UI
  showRecordingUI();
  
  // Start recording timer
  startRecordingTimer();
}

/**
 * Handle recording stop
 */
function handleRecordingStop(audioBlob) {
  console.log('Recording stopped, blob size:', audioBlob.size);
  
  // Update state
  isRecording = false;
  
  // Stop recording timer
  stopRecordingTimer();
  
  // Update UI
  showTranscribingUI();
  
  // Check login status again before transcribing
  if (!isUserLoggedIn()) {
    console.log('Cannot transcribe: User not logged in to TextRad');
    handleTranscriptionError(new Error('You must be logged in to TextRad to transcribe audio'));
    updateAppStatus('not-logged-in');
    return;
  }
  
  // Start transcription
  if (audioBlob && audioBlob.size > 0) {
    transcriptionService.transcribeAudio(audioBlob)
      .catch(error => {
        // Extra error handling for login issues
        if (error.message.includes('logged in') || error.message.includes('login')) {
          updateAppStatus('not-logged-in');
        }
      });
  } else {
    handleTranscriptionError(new Error('No audio recorded'));
  }
}

/**
 * Handle silence detected
 */
function handleSilenceDetected() {
  console.log('Silence detected, auto-stopping recording');
  
  // Update UI
  recordingStatus.textContent = 'Silence detected, stopping...';
  
  // We don't need to call stopRecording() here as the audioRecorder
  // will automatically call onRecordingStop which we handle above
}

/**
 * Handle recording error
 */
function handleRecordingError(error) {
  console.error('Recording error:', error);
  
  // Update state
  isRecording = false;
  
  // Stop recording timer
  stopRecordingTimer();
  
  // Update UI
  recordingStatus.textContent = 'Error: ' + error.message;
  recordButton.disabled = false;
  recordButton.classList.remove('recording');
}

/**
 * Handle transcription start
 */
function handleTranscriptionStart() {
  console.log('Transcription started');
  
  // Update state
  isTranscribing = true;
  
  // Update UI
  recordingStatus.textContent = 'Transcribing...';
}

/**
 * Handle transcription complete
 */
function handleTranscriptionComplete(text) {
  console.log('Transcription complete:', text.substring(0, 50) + '...');
  
  // Update state
  isTranscribing = false;
  
  // Check if we need to append to previous transcription
  let finalText = text;
  const previousTranscription = currentTranscription || '';
  
  console.log('recordMoreButtonClicked flag:', recordMoreButtonClicked);
  console.log('Has previous transcription:', previousTranscription.trim() !== '');
  
  // Only append if we explicitly stored a previous transcription AND the recordMoreButtonClicked flag is true
  if (recordMoreButtonClicked === true && previousTranscription && previousTranscription.trim() !== '') {
    // Append new transcription to the previous one with a space in between
    finalText = previousTranscription + ' ' + text;
    console.log('Appended to previous transcription, result:', finalText.substring(0, 50) + '...');
  } else {
    console.log('Not appending - using just the new transcription');
    // When recordMoreButtonClicked is explicitly false (set by New Dictation), 
    // make sure we use only the new transcription
    finalText = text;
    currentTranscription = '';
  }
  
  // Always reset the recordMoreButtonClicked flag after use
  recordMoreButtonClicked = false;
  
  // Update current transcription
  currentTranscription = finalText;
  
  // Reset the service's stored transcription to avoid duplication
  transcriptionService.setCurrentTranscription('');
  
  // Update UI
  showTranscriptionUI(finalText);
  
  // Notify background script
  chrome.runtime.sendMessage({
    action: 'SET_TRANSCRIPTION',
    transcription: finalText
  });
  
  // Check if auto-send is enabled
  if (autoSendSetting.checked && textradTabs.length > 0) {
    sendTranscription();
  }
}

/**
 * Handle transcription error
 */
function handleTranscriptionError(error) {
  console.error('Transcription error:', error);
  
  // Update state
  isTranscribing = false;
  
  // Different UI handling based on error type
  if (error.message && (
      error.message.includes('audio') || 
      error.message.includes('speech') || 
      error.message.includes('short') ||
      error.message.includes('empty') ||
      error.message.includes('recording'))) {
    // Audio-related error - don't show login prompt
    recordingStatus.textContent = 'Recording issue: ' + error.message;
  } else if (error.message && error.message.includes('timed out')) {
    // Timeout error
    recordingStatus.textContent = 'Timeout: ' + error.message;
  } else if (error.message && (
      error.message.includes('login') || 
      error.message.includes('logged in'))) {
    // Login-related error - update login status
    recordingStatus.textContent = 'Login required';
    updateAppStatus('not-logged-in');
  } else {
    // Generic error
    recordingStatus.textContent = 'Transcription error: ' + error.message;
  }
  
  recordButton.disabled = false;
  hideLoading();
}

/**
 * Start recording timer (simplified, just tracks start time)
 */
function startRecordingTimer() {
  // We still track recording start time for other purposes
  // but we no longer display it
  recordingStartTime = Date.now();
}

/**
 * Stop recording timer
 */
function stopRecordingTimer() {
  if (recordingTimer) {
    clearInterval(recordingTimer);
    recordingTimer = null;
  }
}

/**
 * Show recording UI - Simple approach that always works
 */
function showRecordingUI() {
  // Update record button
  recordButton.disabled = false;
  recordButton.classList.add('recording');
  recordButton.querySelector('i').className = 'fas fa-stop';
  
  // Update status
  recordingStatus.textContent = 'Recording...';
  
  // Hide transcription panel
  transcriptionPanel.classList.add('hidden');
  
  // Ensure recording panel is visible
  document.getElementById('recording-panel').classList.remove('hidden');
  
  // Update mini mode status if it's visible
  updateMiniStatus();
}

/**
 * Show transcribing UI
 */
function showTranscribingUI() {
  // Update record button
  recordButton.disabled = true;
  recordButton.classList.remove('recording');
  recordButton.querySelector('i').className = 'fas fa-microphone';
  
  // Update status
  recordingStatus.textContent = 'Transcribing...';
  
  // Show loading
  showLoading('Transcribing audio...');
  
  // Update mini mode status if it's visible
  updateMiniStatus();
}

/**
 * Show transcription UI
 */
function showTranscriptionUI(text) {
  // Hide loading
  hideLoading();
  
  // Update record button
  recordButton.disabled = false;
  recordButton.classList.remove('recording');
  recordButton.querySelector('i').className = 'fas fa-microphone';
  
  // Update status
  recordingStatus.textContent = 'Ready to record';
  
  // Button is already in its fixed position - no need to move it
  
  // Show recording panel again
  document.getElementById('recording-panel').classList.remove('hidden');
  
  // Show other UI elements
  appStatusPanel.classList.remove('hidden');
  settingsButton.classList.remove('hidden');
  
  // Update transcription text
  transcriptionText.value = text;
  
  // Show transcription panel
  transcriptionPanel.classList.remove('hidden');
  
  // Update mini mode status if it's visible
  updateMiniStatus();
}

/**
 * Show loading overlay
 */
function showLoading(message) {
  loadingText.textContent = message || 'Processing...';
  loadingOverlay.classList.remove('hidden');
}

/**
 * Hide loading overlay
 */
function hideLoading() {
  loadingOverlay.classList.add('hidden');
}

/**
 * Show toast notification
 * @param {string} message - Message to display
 * @param {number} duration - How long to show in ms (default 2000ms)
 */
function showToast(message, duration = 2000) {
  if (!toastNotification || !toastMessage) return;
  
  // Set message
  toastMessage.textContent = message;
  
  // Show toast
  toastNotification.classList.remove('hidden');
  
  // Use setTimeout to ensure the transition works
  setTimeout(() => {
    toastNotification.classList.add('show');
  }, 10);
  
  // Auto hide after duration
  setTimeout(() => {
    // Start fade out
    toastNotification.classList.remove('show');
    toastNotification.classList.add('hide');
    
    // Actually hide after transition completes
    setTimeout(() => {
      toastNotification.classList.remove('hide');
      toastNotification.classList.add('hidden');
    }, 300);
  }, duration);
}

/**
 * Toggle settings panel
 */
function toggleSettings() {
  if (settingsPanel.classList.contains('hidden')) {
    // Show settings
    settingsPanel.classList.remove('hidden');
  } else {
    // Hide settings
    settingsPanel.classList.add('hidden');
  }
}

/**
 * Toggle between expanded and mini modes
 */
function toggleViewMode() {
  // Check current state
  const isExpanded = !expandedContainer.classList.contains('hidden');
  
  if (isExpanded) {
    // Switch to mini mode
    expandedContainer.classList.add('hidden');
    collapsedContainer.classList.remove('hidden');
    document.body.classList.add('mini-mode');
    document.documentElement.classList.add('mini-mode');
    
    // Also notify the background service worker that we're in mini mode
    chrome.runtime.sendMessage({
      action: 'SET_VIEW_MODE',
      mode: 'mini'
    });
    
    // Save in local storage to restore state if popup is closed
    chrome.storage.local.set({ viewMode: 'mini' });
    
    // Force a resize to minimum size
    window.resizeTo(150, 40);
  } else {
    // Switch to expanded mode
    expandedContainer.classList.remove('hidden');
    collapsedContainer.classList.add('hidden');
    document.body.classList.remove('mini-mode');
    document.documentElement.classList.remove('mini-mode');
    
    // Also notify the background service worker that we're in expanded mode
    chrome.runtime.sendMessage({
      action: 'SET_VIEW_MODE',
      mode: 'expanded'
    });
    
    // Save in local storage
    chrome.storage.local.set({ viewMode: 'expanded' });
    
    // Restore normal size
    window.resizeTo(450, 350);
  }
  
  // Update mini mode status indicator based on current state
  updateMiniStatus();
}

/**
 * Handle popup closing
 */
function handlePopupClosing(event) {
  console.log('Popup is closing (beforeunload event)');
  
  // If recording is in progress, we need to do something
  if (isRecording) {
    console.log('Recording was in progress - notifying service worker');
    
    // Notify the background script to maintain recording state
    try {
      chrome.runtime.sendMessage({ 
        action: 'MAINTAIN_RECORDING',
        inProgress: true
      });
    } catch (error) {
      console.error('Failed to notify background script about recording:', error);
    }
  }
}

/**
 * Actually close the popup (when close button is clicked)
 */
function closePopup() {
  // Explicitly stop any recording first
  if (isRecording) {
    stopRecording();
  }
  
  // Then close the window
  window.close();
}

/**
 * Update mini mode status indicator based on current state
 */
function updateMiniStatus() {
  // Remove all classes first
  collapsedStatus.className = 'status-dot';
  
  if (isRecording) {
    // If recording, show red blinking status
    collapsedStatus.classList.add('recording');
    
    // Also update button
    miniRecordButton.classList.add('recording');
    miniRecordButton.querySelector('i').className = 'fas fa-stop';
  } else if (isTranscribing) {
    // If transcribing, show blue blinking status
    collapsedStatus.classList.add('transcribing');
    miniRecordButton.querySelector('i').className = 'fas fa-microphone';
    miniRecordButton.classList.remove('recording');
  } else {
    // Normal state - just show connection status
    miniRecordButton.querySelector('i').className = 'fas fa-microphone';
    miniRecordButton.classList.remove('recording');
    
    // Use the same status as in the app status panel
    if (appStatusIndicator && appStatusIndicator.classList.contains('connected')) {
      collapsedStatus.classList.add('connected');
    } else if (appStatusIndicator && appStatusIndicator.classList.contains('connecting')) {
      collapsedStatus.classList.add('connecting');
    } else {
      collapsedStatus.classList.add('disconnected');
    }
  }
}

/**
 * Update silence timeout value display
 */
function updateSilenceTimeoutValue() {
  const value = silenceTimeoutSetting.value;
  silenceTimeoutValue.textContent = `${value}s`;
}

// Removed transparency functions - using fixed opacity

/**
 * Load settings from storage
 */
async function loadSettings() {
  const settings = await storageManager.getSettings();
  
  // Apply settings
  autoSendSetting.checked = settings.autoSend;
  silenceTimeoutSetting.value = Math.floor(settings.silenceTimeout / 1000);
  
  // Update displayed values
  updateSilenceTimeoutValue();
  
  // Apply settings to services
  if (audioRecorder) {
    audioRecorder.silenceTimeout = settings.silenceTimeout;
  }
}

/**
 * Save settings to storage
 */
async function saveSettings() {
  // Get settings from UI
  const settings = {
    autoSend: autoSendSetting.checked,
    silenceTimeout: parseInt(silenceTimeoutSetting.value) * 1000
  };
  
  // Save settings
  await storageManager.setSettings(settings);
  
  // Fixed transparency - no need to apply anything
  
  // Apply settings to services
  if (audioRecorder) {
    audioRecorder.silenceTimeout = settings.silenceTimeout;
  }
  
  // Hide settings panel
  settingsPanel.classList.add('hidden');
  
  // Show confirmation
  alert('Settings saved');
}

// Initialize popup when DOM is loaded
document.addEventListener('DOMContentLoaded', initialize);